//---------------------------------------------------------------------------
//
//  Asynch.java
//  
//	Demonstrates asynchronous communication using a loopback from
//	FIO4 to FIO5 on a U3 rev 1.30.  On earlier hardware revisions
//  use SDA and SCL.
//
//	Tested with UD driver V3.03.
//  Tested with U3C firmware V1.08?.
//
//  support@labjack.com
//  June 29, 2009
//----------------------------------------------------------------------
//

import java.io.*;
import java.text.*;
import com.labjack.labjackud.LJUD; // importing the LJUD class

public class Asynch {

	// Loading the LJUDJava dll so that we can use the LJUDJava interface. This
	// example assumes that the dll is in the "../../../LJUDJava/" folder.
	static {
		try {
			System.loadLibrary("../../../LJUDJava/LJUDJava");
		} catch (UnsatisfiedLinkError e) {
			System.err.println("Native code library failed to load.\n" + e);
			System.exit(1);
		}
	}

	// This is our simple error handling function that is called after every UD
	// function call. This function displays the errorcode and string
	// description of the error. It also has an iteration input is useful when 
	// processing results in a loop (getfirst/getnext). A stack trace is also
	// displayed so that you can find the line of code where the error occurred.
	public static void ErrorHandler(int Errorcode, int Iteration, Exception excep) {
		byte[] err = new byte[255];

		if (Errorcode != LJUD.LJE_NOERROR) {
			LJUD.ErrorToString(Errorcode, err);
			System.out.println("Error number = " + Errorcode);
			System.out.println("Error string = " + new String(err));
			System.out.println("Iteration = " + Iteration);
			System.out.println("Stack Trace : ");
			excep.printStackTrace();
			if (Errorcode > LJUD.LJE_MIN_GROUP_ERROR) {
				// Quit if this is a group error.
				System.exit(1);
			}
		}
	}

	// Java uses signed bytes but the UD driver requires unsigned bytes therefore these two methods
	// are needed to convert back and forth. For more info see http://www.lykkenborg.no/java/2005/03/how-do-i-read-unsigned-byte-in-java.html
	public static byte signed(int i) 
	{
		return (byte) i;
	}

	public static int unsigned(byte b) 
	{
		return b & 0xFF;
	}

	public void runExample() throws Exception {
		int intErrorcode;
		//long lngGetNextIteration;
		//int[] intIOType={0}, intChannel={0};
		//double[] dblValue={0};
		int[] intHandle={0};

		//double numI2CBytesToWrite;
		double[] numBytes = new double[1];
		byte[] array = new byte[256];


		//Open the LabJack.
		intErrorcode = LJUD.OpenLabJack (LJUD.LJ_dtU3, LJUD.LJ_ctUSB, "1", 1, intHandle);
		ErrorHandler(intErrorcode, 0, new Exception());


		//Start by using the pin_configuration_reset IOType so that all
		//pin assignments are in the factory default condition.
		intErrorcode = LJUD.ePut (intHandle[0], LJUD.LJ_ioPIN_CONFIGURATION_RESET, 0, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());


		// 1 MHz timer clock base.
		intErrorcode = LJUD.ePut(intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chTIMER_CLOCK_BASE, LJUD.LJ_tc1MHZ_DIV, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		// Set clock divisor to 1, so timer clock is 1 MHz.
		intErrorcode = LJUD.ePut(intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chTIMER_CLOCK_DIVISOR, LJUD.LJ_tc1MHZ_DIV, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		// Set timer/counter pin offset to 4. TX and RX appear after any timers and counters on U3
		// hardware rev 1.30.  We have no timers or counters enabled, so TX=FIO4 and RX=FIO5.
		intErrorcode = LJUD.ePut(intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chTIMER_COUNTER_PIN_OFFSET, 4, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		// Set data rate for 9600 bps communication.
		intErrorcode = LJUD.ePut(intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chASYNCH_BAUDFACTOR, 204, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		// Enable UART.
		intErrorcode = LJUD.ePut(intHandle[0], LJUD.LJ_ioASYNCH_COMMUNICATION, LJUD.LJ_chASYNCH_ENABLE, 1, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		// Transmit 2 bytes.
		numBytes[0] = 2;
		array[0] = signed(12);
		array[1] = signed(20);
		intErrorcode = LJUD.eGet(intHandle[0], LJUD.LJ_ioASYNCH_COMMUNICATION, LJUD.LJ_chASYNCH_TX, numBytes, array);
		ErrorHandler(intErrorcode, 0, new Exception());

		// Read 2 bytes.
		numBytes[0] = 9999;  //Dummy values so we can see them change.
		array[0] = signed(222);
		array[1] = signed(222);
		intErrorcode = LJUD.eGet(intHandle[0], LJUD.LJ_ioASYNCH_COMMUNICATION, LJUD.LJ_chASYNCH_RX, numBytes, array);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Display the read data.
		System.out.println("Pre-read buffer size = " + numBytes[0]);
		System.out.println("Read data = " + unsigned(array[0]) + " , " + unsigned(array[1]));

	}

	public static void main(String[] args) {
		try {
			new Asynch().runExample();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

}