/*
 *  U3eFunctions.java
 *  
 *  Demonstrates the UD E-functions with the LabJack U3 with the LJUDJava
 *  JNI wrapper.
 *
 *  support@labjack.com
 *  July 21, 2008
 */

import java.io.*;
import java.text.*;
import com.labjack.labjackud.LJUD; // importing the LJUD class

public class U3eFunctions {

	// Loading the LJUDJava dll so that we can use the LJUDJava interface. This
	// example assumes that the dll is in the "../../../LJUDJava/" folder.
	static {
		try {
			System.loadLibrary("../../../LJUDJava/LJUDJava");
		} catch (UnsatisfiedLinkError e) {
			System.err.println("Native code library failed to load.\n" + e);
			System.exit(1);
		}
	}

	// This is our simple error handling function that is called after every UD
	// function call. This function displays the errorcode and string
	// description of the error. It also has an iteration input is useful when 
	// processing results in a loop (getfirst/getnext). A stack trace is also
	// displayed so that you can find the line of code where the error occurred.
	public static void ErrorHandler(int Errorcode, int Iteration, Exception excep) {
		byte[] err = new byte[255];

		if (Errorcode != LJUD.LJE_NOERROR) {
			LJUD.ErrorToString(Errorcode, err);
			System.out.println("Error number = " + Errorcode);
			System.out.println("Error string = " + new String(err));
			System.out.println("Iteration = " + Iteration);
			System.out.println("Stack Trace : ");
			excep.printStackTrace();
			if (Errorcode > LJUD.LJE_MIN_GROUP_ERROR) {
				// Quit if this is a group error.
				System.exit(1);
			}
		}
	}

	public void runExample() throws Exception {
		int errorcode;
		double[] dblValue = { 0 };
		int[] value = { 0 };
		int[] handle = { 0 };
		int binary;
		int[] aEnableTimers = new int[2];
		int[] aEnableCounters = new int[2];
		int TCPinOffset, timerClockBaseIndex, timerClockDivisor;
		int[] aTimerModes = new int[2];
		double[] adblTimerValues = new double[2];
		int[] aReadTimers = new int[2];
		int[] aUpdateResetTimers = new int[2];
		int[] aReadCounters = new int[2];
		int[] aResetCounters = new int[2];
		double[] adblCounterValues = new double[2];
		double highTime, lowTime, dutyCycle;

		NumberFormat formatter = new DecimalFormat("0.000");

		// Read and display the UD and LJUDJava versions.
		System.out.println("UD Driver Version = " + formatter.format(LJUD.GetDriverVersion()));
		System.out.println("LJUD JNI Version = " + formatter.format(LJUD.LJUDJAVA_VERSION) + "\n");

		// Open the first found LabJack U3.
		errorcode = LJUD.OpenLabJack(LJUD.LJ_dtU3, LJUD.LJ_ctUSB, "1", 1, handle);
		ErrorHandler(errorcode, 0, new Exception());

		// Start by using the pin_configuration_reset IOType so that all
		// pin assignments are in the factory default condition.
		errorcode = LJUD.ePut(handle[0], LJUD.LJ_ioPIN_CONFIGURATION_RESET, 0, 0, 0);
		ErrorHandler(errorcode, 0, new Exception());

		// Take a measurement from AIN3.
		binary = 0;
		errorcode = LJUD.eAIN(handle[0], 3, 31, dblValue, -1, -1, -1, binary, 0, 0);
		ErrorHandler(errorcode, 0, new Exception());
		System.out.println("AIN3 = " + dblValue[0]);

		// Set DAC0 to 3.0 volts.
		dblValue[0] = 3.0;
		binary = 0;
		errorcode = LJUD.eDAC(handle[0], 0, dblValue[0], binary, 0, 0);
		ErrorHandler(errorcode, 0, new Exception());
		System.out.println("DAC0 set to " + dblValue[0] + " volts");

		// Read state of FIO4.
		errorcode = LJUD.eDI(handle[0], 4, value);
		ErrorHandler(errorcode, 0, new Exception());
		System.out.println("FIO4 = " + value[0]);

		// Set the state of FIO7.
		value[0] = 1;
		errorcode = LJUD.eDO(handle[0], 7, value[0]);
		ErrorHandler(errorcode, 0, new Exception());
		System.out.println("FIO7 set to = " + value[0] + "\n");

		// Timers and Counters example.
		// First, a call to eTCConfig. Fill the arrays with the desired values,
		// then make the call.
		aEnableTimers[0] = 1; // Enable Timer0 (uses FIO4).
		aEnableTimers[1] = 1; // Enable Timer1 (uses FIO5).
		aEnableCounters[0] = 0; // Disable Counter0.
		aEnableCounters[1] = 1; // Enable Counter1 (uses FIO6).
		TCPinOffset = 4; // Offset is 4, so timers/counters start at FIO4.
		timerClockBaseIndex = LJUD.LJ_tc48MHZ_DIV; // Base clock is 48 MHz with divisor support, so Counter0 is not available.
		// timerClockBaseIndex = LJUD.LJ_tc24MHZ_DIV; //Use this line instead for hardware rev 1.20.
		timerClockDivisor = 48; // Thus timer clock is 1 MHz.
		// timerClockDivisor = 24; // Use this line instead for hardware rev 1.20.
		aTimerModes[0] = LJUD.LJ_tmPWM8;	// Timer0 is 8-bit PWM output. Frequency is 1M/256 = 977 Hz.
		aTimerModes[1] = LJUD.LJ_tmDUTYCYCLE; // Timer1 is duty cyle input.
		adblTimerValues[0] = 16384; // Set PWM8 duty-cycle to 75%.
		adblTimerValues[1] = 0;
		errorcode = LJUD.eTCConfig(handle[0], aEnableTimers, aEnableCounters,
				TCPinOffset, timerClockBaseIndex, timerClockDivisor,
				aTimerModes, adblTimerValues, 0, 0);
		ErrorHandler(errorcode, 0, new Exception());
		System.out.println("Timers and Counters enabled.\n");

		Thread.sleep(1000); // Wait 1 second.

		// Now, a call to eTCValues.
		aReadTimers[0] = 0; // Don't read Timer0 (output timer).
		aReadTimers[1] = 1; // Read Timer1.
		aUpdateResetTimers[0] = 1; // Update Timer0.
		aUpdateResetTimers[1] = 1; // Reset Timer1.
		aReadCounters[0] = 0;
		aReadCounters[1] = 1; // Read Counter1.
		aResetCounters[0] = 0;
		aResetCounters[1] = 1; // Reset Counter1.
		adblTimerValues[0] = 32768; // Change Timer0 duty-cycle to 50%.
		adblTimerValues[1] = 0;
		errorcode = LJUD.eTCValues(handle[0], aReadTimers, aUpdateResetTimers,
				aReadCounters, aResetCounters, adblTimerValues,
				adblCounterValues, 0, 0);
		ErrorHandler(errorcode, 0, new Exception());
		System.out.println("Timer1 value = " + formatter.format(adblTimerValues[1]));
		System.out.println("Counter1 value = "	+ formatter.format(adblCounterValues[1]));

		// Convert Timer1 value to duty-cycle percentage.
		// High time is LSW
		highTime = (double) (((long) adblTimerValues[1]) % (65536));
		// Low time is MSW
		lowTime = (double) (((long) adblTimerValues[1]) / (65536));
		// Calculate the duty cycle percentage.
		dutyCycle = (((highTime + lowTime) != 0) ? (100 * highTime / (highTime + lowTime)) : 0);
		formatter = new DecimalFormat("0.0");
		System.out.println("\nHigh clicks Timer1 = " + formatter.format(highTime));
		System.out.println("Low clicks Timer1 = " + formatter.format(lowTime));
		System.out.println("Duty cycle Timer1 = " + formatter.format(dutyCycle));

		// Disable all timers and counters.
		aEnableTimers[0] = 0;
		aEnableTimers[1] = 0;
		aEnableCounters[0] = 0;
		aEnableCounters[1] = 0;
		errorcode = LJUD.eTCConfig(handle[0], aEnableTimers, aEnableCounters,
				4, timerClockBaseIndex, timerClockDivisor, aTimerModes,
				adblTimerValues, 0, 0);
		ErrorHandler(errorcode, 0, new Exception());
	}

	public U3eFunctions() {
	}

	public static void main(String[] args) {
		try {
			new U3eFunctions().runExample();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

}