//---------------------------------------------------------------------------
//
//  U6ei1050.java
//  
//  Demonstrates talking to 1 or 2 EI-1050 probes.
//
//  support@labjack.com
//  June 25, 2009
//----------------------------------------------------------------------
//

import java.io.*;
import java.text.*;
import com.labjack.labjackud.*; // importing the LJUD class

public class EI1050 {

	// Loading the LJUDJava dll so that we can use the LJUDJava interface. This
	// example assumes that the dll is in the "../../../LJUDJava/" folder.
	static {
		try {
			System.loadLibrary("../../../LJUDJava/LJUDJava");
		} catch (UnsatisfiedLinkError e) {
			System.err.println("Native code library failed to load.\n" + e);
			System.exit(1);
		}
	}

	// This is our simple error handling function that is called after every UD
	// function call. This function displays the errorcode and string
	// description of the error. It also has an iteration input is useful when 
	// processing results in a loop (getfirst/getnext). A stack trace is also
	// displayed so that you can find the line of code where the error occurred.
	public static void ErrorHandler(int Errorcode, int Iteration, Exception excep) {
		byte[] err = new byte[255];

		if (Errorcode != LJUD.LJE_NOERROR) {
			LJUD.ErrorToString(Errorcode, err);
			System.out.println("Error number = " + Errorcode);
			System.out.println("Error string = " + new String(err));
			System.out.println("Iteration = " + Iteration);
			System.out.println("Stack Trace : ");
			excep.printStackTrace();
			if (Errorcode > LJUD.LJE_MIN_GROUP_ERROR) {
				// Quit if this is a group error.
				System.exit(1);
			}
		}
	}

	public void runExample() throws Exception {
		int intErrorcode;
		double[] dblValue={0};
		int[] intHandle={0};
		char ch;

		//Open the first found LabJack U3.
		intErrorcode = LJUD.OpenLabJack (LJUD.LJ_dtU3, LJUD.LJ_ctUSB, "1", 1, intHandle);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Start by using the pin_configuration_reset IOType so that all
		//pin assignments are in the factory default condition.
		intErrorcode = LJUD.ePut (intHandle[0], LJUD.LJ_ioPIN_CONFIGURATION_RESET, 0, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());



		//Set the Data line to FIO4, which is the default anyway. 
		intErrorcode = LJUD.ePut (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chSHT_DATA_CHANNEL, 4, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Set the Clock line to FIO5, which is the default anyway. 
		intErrorcode = LJUD.ePut (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chSHT_CLOCK_CHANNEL, 5, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Set FIO6 to output-high to provide Math.power to the EI-1050s. 
		intErrorcode = LJUD.ePut (intHandle[0], LJUD.LJ_ioPUT_DIGITAL_BIT, 6, 1, 0);
		ErrorHandler(intErrorcode, 0, new Exception());


	
		/*
		//Use this code if only a single EI-1050 is connected.
		//	Connections for one probe:
		//	Red (Power)         FIO6
		//	Black (Ground)      GND
		//	Green (Data)        FIO4
		//	White (Clock)       FIO5
		//	Brown (Enable)      FIO6

		//Now, an add/go/get block to get the temp  humidity at the same time.
		//Request a temperature reading from the EI-1050.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioSHT_GET_READING, LJUD.LJ_chSHT_TEMP, 0, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Request a humidity reading from the EI-1050.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioSHT_GET_READING, LJUD.LJ_chSHT_RH, 0, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Execute the requests.  Will take about 0.5 seconds with a USB high-high
		//or Ethernet connection, and about 1.5 seconds with a normal USB connection.
		intErrorcode = LJUD.GoOne (intHandle[0]);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Get the temperature reading.
		intErrorcode = LJUD.GetResult (intHandle[0], LJUD.LJ_ioSHT_GET_READING, LJUD.LJ_chSHT_TEMP, dblValue);
			ErrorHandler(intErrorcode, 0, new Exception());
		System.out.println("Temp Probe A = " + dblValue[0] + " deg K");
		System.out.println("Temp Probe A = " + (dblValue[0]-273.15) + " deg C");
		System.out.println("Temp Probe A = " +  (((dblValue[0]-273.15)*1.8)+32) + " deg F");

		//Get the humidity reading.
		intErrorcode = LJUD.GetResult (intHandle[0], LJUD.LJ_ioSHT_GET_READING, LJUD.LJ_chSHT_RH, dblValue);
			ErrorHandler(intErrorcode, 0, new Exception());
		System.out.println("RH Probe A = " + dblValue[0] + " percent\n");

		//End of single probe code.
		*/


		///*
		 //Use this code if two EI-1050 probes are connected.
		 //	Connections for both probes:
		 //	Red (Power)         FIO6
		 //	Black (Ground)      GND
		 //	Green (Data)        FIO4
		 //	White (Clock)       FIO5
		 //
		 //	Probe A:
		 //	Brown (Enable)    FIO7
		 //
		 //	Probe B:
		 //	Brown (Enable)    DAC0

		 //Set FIO7 to output-low to disable probe A. 
		 intErrorcode = LJUD.ePut (intHandle[0], LJUD.LJ_ioPUT_DIGITAL_BIT, 7, 0, 0);
		 ErrorHandler(intErrorcode, 0, new Exception());

		 //Set DAC0 to 0 volts to disable probe B.
		 intErrorcode = LJUD.ePut (intHandle[0], LJUD.LJ_ioPUT_DAC, 0, 0.0, 0);
		 ErrorHandler(intErrorcode, 0, new Exception());

		 //Set FIO7 to output-high to enable probe A. 
		 intErrorcode = LJUD.ePut (intHandle[0], LJUD.LJ_ioPUT_DIGITAL_BIT, 7, 1, 0);
		 ErrorHandler(intErrorcode, 0, new Exception());

		 //Now, an add/go/get block to get the temp  humidity at the same time.
		 //Request a temperature reading from the EI-1050.
		 intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioSHT_GET_READING, LJUD.LJ_chSHT_TEMP, 0, 0, 0);
		 ErrorHandler(intErrorcode, 0, new Exception());

		 //Request a humidity reading from the EI-1050.
		 intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioSHT_GET_READING, LJUD.LJ_chSHT_RH, 0, 0, 0);
		 ErrorHandler(intErrorcode, 0, new Exception());

		 //Execute the requests.  Will take about 0.5 seconds with a USB high-high
		 //or Ethernet connection, and about 1.5 seconds with a normal USB connection.
		 intErrorcode = LJUD.GoOne (intHandle[0]);
		 ErrorHandler(intErrorcode, 0, new Exception());

		 //Get the temperature reading.
		 intErrorcode = LJUD.GetResult (intHandle[0], LJUD.LJ_ioSHT_GET_READING, LJUD.LJ_chSHT_TEMP, dblValue);
		 ErrorHandler(intErrorcode, 0, new Exception());
		 System.out.println("Temp Probe A = " + dblValue[0] + " deg K");
		 System.out.println("Temp Probe A = " + (dblValue[0]-273.15) + " deg C");
		 System.out.println("Temp Probe A = " + (((dblValue[0]-273.15)*1.8)+32) + " deg F");

		 //Get the humidity reading.
		 intErrorcode = LJUD.GetResult (intHandle[0], LJUD.LJ_ioSHT_GET_READING, LJUD.LJ_chSHT_RH, dblValue);
		 ErrorHandler(intErrorcode, 0, new Exception());
		 System.out.println("RH Probe A = " + dblValue[0] + " percent\n");

		 //Set FIO7 to output-low to disable probe A. 
		 intErrorcode = LJUD.ePut (intHandle[0], LJUD.LJ_ioPUT_DIGITAL_BIT, 7, 0, 0);
		 ErrorHandler(intErrorcode, 0, new Exception());

		 //Set DAC0 to 3.3 volts to enable probe B.
		 intErrorcode = LJUD.ePut (intHandle[0], LJUD.LJ_ioPUT_DAC, 0, 3.3, 0);
		 ErrorHandler(intErrorcode, 0, new Exception());

		 //Since the DACs on the U3 are slower than the communication speed,
		 //we put a delay here to make sure the DAC has time to rise to 3.3 volts
		 //before communicating with the EI-1050.
		 Thread.sleep(30);  //Wait 30 ms.

		 //Now, an add/go/get block to get the temp  humidity at the same time.
		 //Request a temperature reading from the EI-1050.
		 intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioSHT_GET_READING, LJUD.LJ_chSHT_TEMP, 0, 0, 0);
		 ErrorHandler(intErrorcode, 0, new Exception());

		 //Request a humidity reading from the EI-1050.
		 intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioSHT_GET_READING, LJUD.LJ_chSHT_RH, 0, 0, 0);
		 ErrorHandler(intErrorcode, 0, new Exception());

		 //Execute the requests.  Will take about 0.5 seconds with a USB high-high
		 //or Ethernet connection, and about 1.5 seconds with a normal USB connection.
		 intErrorcode = LJUD.GoOne (intHandle[0]);
		 ErrorHandler(intErrorcode, 0, new Exception());

		 //Get the temperature reading.
		 intErrorcode = LJUD.GetResult (intHandle[0], LJUD.LJ_ioSHT_GET_READING, LJUD.LJ_chSHT_TEMP, dblValue);
		 ErrorHandler(intErrorcode, 0, new Exception());
		 System.out.println("Temp Probe B = " + dblValue[0] + " deg K");
		 System.out.println("Temp Probe B = " + (dblValue[0]-273.15) + " deg C");
		 System.out.println("Temp Probe B = " + (((dblValue[0]-273.15)*1.8)+32) + " deg F");

		 //Get the humidity reading.
		 intErrorcode = LJUD.GetResult (intHandle[0], LJUD.LJ_ioSHT_GET_READING, LJUD.LJ_chSHT_RH, dblValue);
		 ErrorHandler(intErrorcode, 0, new Exception());
		 System.out.println("RH Probe B = " + dblValue[0] + " percent\n");

		 //Set DAC0 to 0 volts to disable probe B.
		 intErrorcode = LJUD.ePut (intHandle[0], LJUD.LJ_ioPUT_DAC, 0, 0.0, 0);
		 ErrorHandler(intErrorcode, 0, new Exception());
			
		 //If we were going to loop and talk to probe A next, we would
		 //want a delay here to make sure the DAC falls to 0 volts
		 //before enabling probe A.
		 Thread.sleep(30);  //Wait 30 ms.

		 //End of dual probe code.
		 //*/
	}

	public static void main(String[] args) {
		try {
			new EI1050().runExample();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

}