/*
 *  U3simple.java
 *  
 *  Basic command/response U3 example using the UD driver with the LJUDJava
 *  JNI wrapper.
 *
 *  support@labjack.com
 *  July 21, 2008
 */

import java.io.*;
import java.text.*;
import com.labjack.labjackud.LJUD; // importing the LJUD class

public class U3Simple {

	// Loading the LJUDJava dll so that we can use the LJUDJava interface. This
	// example assumes that the dll is in the "../../../LJUDJava/" folder.
	static {
		try {
			System.loadLibrary("../../../LJUDJava/LJUDJava");
		} catch (UnsatisfiedLinkError e) {
			System.err.println("Native code library failed to load.\n" + e);
			System.exit(1);
		}
	}

	// This is our simple error handling function that is called after every UD
	// function call. This function displays the errorcode and string
	// description of the error. It also has an iteration input is useful when 
	// processing results in a loop (getfirst/getnext). A stack trace is also
	// displayed so that you can find the line of code where the error occurred.
	public static void ErrorHandler(int Errorcode, int Iteration, Exception excep) {
		byte[] err = new byte[255];

		if (Errorcode != LJUD.LJE_NOERROR) {
			LJUD.ErrorToString(Errorcode, err);
			System.out.println("Error number = " + Errorcode);
			System.out.println("Error string = " + new String(err));
			System.out.println("Iteration = " + Iteration);
			System.out.println("Stack Trace : ");
			excep.printStackTrace();
			if (Errorcode > LJUD.LJE_MIN_GROUP_ERROR) {
				// Quit if this is a group error.
				System.exit(1);
			}
		}
	}

	public void runExample() throws Exception {
		int Errorcode;
		int GetNextIteration;
		int[] IOType = { 0 };
		int[] Channel = { 0 };
		double[] Value = { 0 };
		double Value0 = 9999, Value1 = 9999, Value2 = 9999;
		double ValueDIBit = 9999, ValueDIPort = 9999, ValueCounter = 9999;
		int[] Handle = { 0 };
		char ch;
		int[] iZero = { 0 };
		double[] dZero = { 0 };

		NumberFormat formatter = new DecimalFormat("0.000");
		BufferedReader keyboard = new BufferedReader(
				new InputStreamReader(System.in));

		// Read and display the UD and LJUDJava versions.
		System.out.println("UD Driver Version = " + formatter.format(LJUD.GetDriverVersion()));
		System.out.println("LJUD JNI Version = " + formatter.format(LJUD.LJUDJAVA_VERSION) + "\n");

		// Open the first found LabJack U3.
		Errorcode = LJUD.OpenLabJack(LJUD.LJ_dtU3, LJUD.LJ_ctUSB, "1", 1, Handle);
		ErrorHandler(Errorcode, 0, new Exception());

		// Start by using the pin_configuration_reset IOType so that all
		// pin assignments are in the factory default condition.
		Errorcode = LJUD.ePut(Handle[0], LJUD.LJ_ioPIN_CONFIGURATION_RESET, 0, 0, 0);
		ErrorHandler(Errorcode, 0, new Exception());

		// First some configuration commands. These will be done with the ePut
		// function which combines the add/go/get into a single call.

		// Configure FIO0 and FIO3 as analog, all else as digital. That means we
		// will start from channel 0 and update all 16 flexible bits. We will
		// pass a value of b0000000000001111 or d15.
		Errorcode = LJUD.ePut(Handle[0], LJUD.LJ_ioPUT_ANALOG_ENABLE_PORT, 0, 15, 16);
		ErrorHandler(Errorcode, 0, new Exception());

		// Set the timer/counter pin offset to 7, which will put the first
		// timer/counter on FIO7.
		Errorcode = LJUD.ePut(Handle[0], LJUD.LJ_ioPUT_CONFIG,
				LJUD.LJ_chTIMER_COUNTER_PIN_OFFSET, 7, 0);
		ErrorHandler(Errorcode, 0, new Exception());

		// Enable Counter1 (FIO7).
		Errorcode = LJUD.ePut(Handle[0], LJUD.LJ_ioPUT_COUNTER_ENABLE, 1, 1, 0);
		ErrorHandler(Errorcode, 0, new Exception());


		// The following commands will use the add-go-get method to group
		// multiple requests into a single low-level function.

		// Request a single-ended reading from AIN0.
		Errorcode = LJUD.AddRequest(Handle[0], LJUD.LJ_ioGET_AIN, 0, 0, 0, 0);
		ErrorHandler(Errorcode, 0, new Exception());

		// Request a single-ended reading from AIN1.
		Errorcode = LJUD.AddRequest(Handle[0], LJUD.LJ_ioGET_AIN, 1, 0, 0, 0);
		ErrorHandler(Errorcode, 0, new Exception());

		// Request a reading from AIN2 using the Special range.
		Errorcode = LJUD.AddRequest(Handle[0], LJUD.LJ_ioGET_AIN_DIFF, 2, 0, 32, 0);
		ErrorHandler(Errorcode, 0, new Exception());

		// Set DAC0 to 3.5 volts.
		Errorcode = LJUD.AddRequest(Handle[0], LJUD.LJ_ioPUT_DAC, 0, 3.5, 0, 0);
		ErrorHandler(Errorcode, 0, new Exception());
		
		// Set digital output FIO4 to output-high.
		Errorcode = LJUD.AddRequest(Handle[0], LJUD.LJ_ioPUT_DIGITAL_BIT, 4, 1,	0, 0);
		ErrorHandler(Errorcode, 0, new Exception());

		// Read digital input FIO5.
		Errorcode = LJUD.AddRequest(Handle[0], LJUD.LJ_ioGET_DIGITAL_BIT, 5, 0, 0, 0);
		ErrorHandler(Errorcode, 0, new Exception());
	
		// Read digital inputs FIO5 through FIO6.
		Errorcode = LJUD.AddRequest(Handle[0], LJUD.LJ_ioGET_DIGITAL_PORT, 5, 0, 2, 0);
		ErrorHandler(Errorcode, 0, new Exception());

		// Request the value of Counter1.
		Errorcode = LJUD.AddRequest(Handle[0], LJUD.LJ_ioGET_COUNTER, 1, 0, 0, 0);
		ErrorHandler(Errorcode, 0, new Exception());

		while (true) {
			// Execute the requests.
			Errorcode = LJUD.GoOne(Handle[0]);
			ErrorHandler(Errorcode, 0, new Exception());

			// Get all the results. The input measurement results are stored.
			// All other results are for configuration or output requests so 
			// we are just checking whether there was an error.
			Errorcode = LJUD.GetFirstResult(Handle[0], IOType, Channel, Value, iZero, dZero);
			ErrorHandler(Errorcode, 0, new Exception());

			GetNextIteration = 0; // Used by the error handling function.

			while (Errorcode < LJUD.LJE_MIN_GROUP_ERROR) {
				if (IOType[0] == LJUD.LJ_ioGET_AIN) {
					if (Channel[0] == 0)
						Value0 = Value[0];
					if (Channel[0] == 1)
						Value1 = Value[0];
				}

				if (IOType[0] == LJUD.LJ_ioGET_AIN_DIFF)
					Value2 = Value[0];

				if (IOType[0] == LJUD.LJ_ioGET_DIGITAL_BIT)
					ValueDIBit = Value[0];

				if (IOType[0] == LJUD.LJ_ioGET_DIGITAL_PORT)
					ValueDIPort = Value[0];

				if (IOType[0] == LJUD.LJ_ioGET_COUNTER)
					ValueCounter = Value[0];

				Errorcode = LJUD.GetNextResult(Handle[0], IOType, Channel, Value, iZero, dZero);
				if (Errorcode != LJUD.LJE_NO_MORE_DATA_AVAILABLE)
					ErrorHandler(Errorcode, GetNextIteration, new Exception());

				GetNextIteration++;
			}

			System.out.println("AIN0 = " + formatter.format(Value0));
			System.out.println("AIN1 = " + formatter.format(Value1));
			System.out.println("AIN2 = " + formatter.format(Value2));
			System.out.println("FIO5 = " + formatter.format(ValueDIBit));
			System.out.println("FIO5-FIO6 = " + ValueDIPort);	// Will read 3 (binary 11) if both lines are pulled-high as normal.
			System.out.println("Counter1 (FIO7) " + ValueCounter);

			System.out.println("\nPress Enter to go again or (q) and Enter to quit\n");
			ch = (char) keyboard.read();
			if (ch == 'q')
				return;

			System.out.println("\nDone");
		}
	}

	public U3Simple() {
	}

	public static void main(String[] args) {
		try {
			new U3Simple().runExample();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

}