/*
 *  U3SimpleStream.java
 *  
 *  2-channel stream on the U3 using the UD driver with the LJUDJava JNI
 *  wrapper.
 *
 *  support@labjack.com
 *  July 21, 2008
 */

import java.io.*;
import java.text.*;
import com.labjack.labjackud.LJUD; // importing the LJUD class

public class U3SimpleStream {

	// Loading the LJUDJava dll so that we can use the LJUDJava interface. This
	// example assumes that the dll is in the "../../../LJUDJava/" folder.
	static {
		try {
			System.loadLibrary("../../../LJUDJava/LJUDJava");
		} catch (UnsatisfiedLinkError e) {
			System.err.println("Native code library failed to load.\n" + e);
			System.exit(1);
		}
	}

	// This is our simple error handling function that is called after every UD
	// function call. This function displays the errorcode and string
	// description of the error. It also has an iteration input is useful when
	// processing results in a loop (getfirst/getnext). A stack trace is also 
	// displayed so that you can find the line of code where the error occurred.
	public static void ErrorHandler(int Errorcode, int Iteration, Exception excep) {
		byte[] err = new byte[255];

		if (Errorcode != LJUD.LJE_NOERROR) {
			LJUD.ErrorToString(Errorcode, err);
			System.out.println("Error number = " + Errorcode);
			System.out.println("Error string = " + new String(err));
			System.out.println("Iteration = " + Iteration);
			System.out.println("Stack Trace : ");
			excep.printStackTrace();
			if (Errorcode > LJUD.LJE_MIN_GROUP_ERROR) {
				// Quit if this is a group error.
				System.exit(1);
			}
		}
	}

	private boolean kbhit() {
		try {
			return (System.in.available() != 0);
		} catch (Exception e) {
		}
		return false;
	}

	public void runExample() throws Exception {
		int Errorcode;
		int[] Handle = { 0 };
		int GetNextIteration;
		int i = 0, k = 0;
		int[] IOType = { 0 };
		int[] Channel = { 0 };
		double[] Value = { 0 };
		double[] Backlog = { 0 };
		double scanRate = 1000;
		long delayms = 1000;
		double numScans = 2000; // 2x the expected # of scans (2*scanRate*delayms/1000)
		double[] numScansRequested = { 0 };
		double[] adblData = new double[4000]; 	// Max buffer size
												// (#channels*numScansRequested)
		int[] iZero = { 0 };
		double[] dZero = { 0 };
		NumberFormat formatter = new DecimalFormat("0.000");

		// Read and display the UD and LJUDJava versions.
		System.out.println("UD Driver Version = " + formatter.format(LJUD.GetDriverVersion()));
		System.out.println("LJUDJava JNI Version = " + formatter.format(LJUD.LJUDJAVA_VERSION) + "\n");

		// Open the first found LabJack U3.
		Errorcode = LJUD.OpenLabJack(LJUD.LJ_dtU3, LJUD.LJ_ctUSB, "1", 1, Handle);
		ErrorHandler(Errorcode, 0, new Exception());

		// Read and display the hardware version of this U3.
		Errorcode = LJUD.eGet(Handle[0], LJUD.LJ_ioGET_CONFIG, LJUD.LJ_chHARDWARE_VERSION, Value, 0);
		System.out.println("U3 Hardware Version = " + formatter.format(Value[0]) + "\n");
		ErrorHandler(Errorcode, 0, new Exception());

		// Read and display the firmware version of this U3.
		Errorcode = LJUD.eGet(Handle[0], LJUD.LJ_ioGET_CONFIG, LJUD.LJ_chFIRMWARE_VERSION, Value, 0);
		System.out.println("U3 Firmware Version = "	+ formatter.format(Value[0]) + "\n");
		ErrorHandler(Errorcode, 0, new Exception());

		// Start by using the pin_configuration_reset IOType so that all
		// pin assignments are in the factory default condition.
		Errorcode = LJUD.ePut(Handle[0], LJUD.LJ_ioPIN_CONFIGURATION_RESET, 0, 0, 0);
		ErrorHandler(Errorcode, 0, new Exception());

		// Configure FIO0 and FIO1 as analog, all else as digital. That means we
		// will start from channel 0 and update all 16 flexible bits. We will
		// pass a value of b0000000000000011 or d3.
		Errorcode = LJUD.ePut(Handle[0], LJUD.LJ_ioPUT_ANALOG_ENABLE_PORT, 0, 3, 16);
		ErrorHandler(Errorcode, 0, new Exception());


		// Configure the stream:
		// Set the scan rate.
		Errorcode = LJUD.AddRequest(Handle[0], LJUD.LJ_ioPUT_CONFIG,
				LJUD.LJ_chSTREAM_SCAN_FREQUENCY, scanRate, 0, 0);
		ErrorHandler(Errorcode, 0, new Exception());
		// Give the driver a 5 second buffer (scanRate * 2 channels * 5 seconds).
		Errorcode = LJUD.AddRequest(Handle[0], LJUD.LJ_ioPUT_CONFIG,
				LJUD.LJ_chSTREAM_BUFFER_SIZE, scanRate * 2 * 5, 0, 0);
		ErrorHandler(Errorcode, 0, new Exception());
		// Configure reads to retrieve whatever data is available without
		// waiting (wait mode LJ_swNONE).  See comments below to change this 
		// program to use LJ_swSLEEP mode.
		Errorcode = LJUD.AddRequest(Handle[0], LJUD.LJ_ioPUT_CONFIG,
				LJUD.LJ_chSTREAM_WAIT_MODE, LJUD.LJ_swNONE, 0, 0);
		ErrorHandler(Errorcode, 0, new Exception());
		// Define the scan list as AIN0 then FIOEIO.
		Errorcode = LJUD.AddRequest(Handle[0], LJUD.LJ_ioCLEAR_STREAM_CHANNELS,	0, 0, 0, 0);
		ErrorHandler(Errorcode, 0, new Exception());
		Errorcode = LJUD.AddRequest(Handle[0], LJUD.LJ_ioADD_STREAM_CHANNEL, 0, 0, 0, 0);
		ErrorHandler(Errorcode, 0, new Exception());
		Errorcode = LJUD.AddRequest(Handle[0], LJUD.LJ_ioADD_STREAM_CHANNEL_DIFF, 1, 0, 32, 0);
		ErrorHandler(Errorcode, 0, new Exception());

		// Execute the list of requests.
		Errorcode = LJUD.GoOne(Handle[0]);
		ErrorHandler(Errorcode, 0, new Exception());

		// Get all the results just to check for errors.
		Errorcode = LJUD.GetFirstResult(Handle[0], IOType, Channel, Value, iZero, dZero);
		ErrorHandler(Errorcode, 0, new Exception());
		GetNextIteration = 0; // Used by the error handling function.
		while (Errorcode < LJUD.LJE_MIN_GROUP_ERROR) {
			Errorcode = LJUD.GetNextResult(Handle[0], IOType, Channel, Value, iZero, dZero);
			if (Errorcode != LJUD.LJE_NO_MORE_DATA_AVAILABLE)
				ErrorHandler(Errorcode, GetNextIteration, new Exception());

			GetNextIteration++;
		}

		// Start the stream.
		Errorcode = LJUD.eGet(Handle[0], LJUD.LJ_ioSTART_STREAM, 0, Value, 0);
		ErrorHandler(Errorcode, 0, new Exception());

		// The actual scan rate is dependent on how the desired scan rate
		// divides into the LabJack clock. The actual scan rate is returned
		// in the value parameter from the start stream command.
		System.out.println("Actual Scan Rate = " + formatter.format(Value[0]));
		System.out.println("Actual Sample Rate = " + formatter.format(2 * Value[0]));

		// Read data
		while (!kbhit()) // Loop will run until Enter is hit
		{
			// Since we are using wait mode LJ_swNONE, we will wait a little,
			// then read however much data is available. Thus this delay will 
			// control how fast the program loops and how much data is read each
			// loop. An alternative common method is to use wait mode LJ_swSLEEP
			// where the stream read waits for a certain number of scans. In such
			// a case you would not have a delay here, since the stream read will
			// actually control how fast the program loops.
			//
			// To change this program to use sleep mode,
			// -change numScans to the actual number of scans desired per read,
			// -change wait mode addrequest value to LJ_swSLEEP,
			// -comment out the following Sleep command.

			Thread.sleep(delayms); // Remove if using LJ_swSLEEP.

			// init array so we can easily tell if it has changed
			for (k = 0; k < numScans * 2; k++)
				adblData[k] = 9999.0;

			// Read the data. We will request twice the number we expect, to
			// make sure we get everything that is available.
			// Note that the array we pass must be sized to hold enough SAMPLES,
			// and the Value we pass specifies the number of SCANS to read.
			numScansRequested[0] = numScans;
			Errorcode = LJUD.eGet(Handle[0], LJUD.LJ_ioGET_STREAM_DATA,
					LJUD.LJ_chALL_CHANNELS, numScansRequested, adblData);
			// The displays the number of scans that were actually read.
			System.out.println("\nIteration # " + i);
			System.out.println("Number read = " + (long) numScansRequested[0]);
			// This displays just the first scan.
			System.out.println("First scan = " + formatter.format(adblData[0])
					+ ", " + formatter.format(adblData[1]));
			ErrorHandler(Errorcode, 0, new Exception());
			// Retrieve the current backlog. The UD driver retrieves stream data
			// from the U3 in the background, but if the computer is too slow
			// for some reason the driver might not be able to read the data as
			// fast as the U3 is acquiring it, and thus there will be data left
			// over in the U3 buffer.
			Errorcode = LJUD.eGet(Handle[0], LJUD.LJ_ioGET_CONFIG,
					LJUD.LJ_chSTREAM_BACKLOG_COMM, Backlog, 0);
			System.out.println("Comm Backlog = " + (long) Backlog[0]);
			i++;
		}

		// Stop the stream
		Errorcode = LJUD.eGet(Handle[0], LJUD.LJ_ioSTOP_STREAM, 0, dZero, 0);
		ErrorHandler(Errorcode, 0, new Exception());

		System.out.println("\nDone");
	}

	public U3SimpleStream() {
	}

	public static void main(String[] args) {
		try {
			new U3SimpleStream().runExample();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

}