//---------------------------------------------------------------------------
//
//  Thermocouple.java
// 
//	Demonstrates using the TCVoltsToTemp to yield temperature readings for
//	a thermocouple connected to a U3 via a LJTick-InAmp
//
//  Setup:
//  TC plus to INA+.
//	TC minus to INA-.
//	10k resistor from INA- to GND on LJTIA (a short is also acceptable in most cases).
//	LJTIA connected to AIN0/AIN1 block on U3.
//	LJTIA offset set to 0.4 volts.
//	LJTIA channel A gain set to 51.
//
//  For the best accuracy, we recommend doing a quick offset calibration as
//	described in the comments below, and using an external cold-junction
//	temperature sensor rather than the internal U3 temp sensor.  The internal
//	temp sensor is often sufficient, but an external sensor, such as 
//	the LM34CAZ (national.com) or the EI-1034 (labjack.com), placed near
//	the LJTIA, can provide better CJC, particularly when the U3 itself
//	is subjected to varying temperatures.
//
//  support@labjack.com
//  June 26, 2009
//----------------------------------------------------------------------

import java.io.*;
import java.text.*;
import java.util.Scanner;
import com.labjack.labjackud.LJUD; // importing the LJUD class

public class Thermocouple {

	public static void main(String[] args) 
	{
		try 
		{
			new Thermocouple().runExample();
		} 
		catch (Exception ex) 
		{
			ex.printStackTrace();
		}
	}

	// Loading the LJUDJava dll so that we can use the LJUDJava interface. This
	// example assumes that the dll is in the "../../../LJUDJava/" folder.
	static {
		try {
			System.loadLibrary("../../../LJUDJava/LJUDJava");
		} catch (UnsatisfiedLinkError e) {
			System.err.println("Native code library failed to load.\n" + e);
			System.exit(1);
		}
	}

	// This is our simple error handling function that is called after every UD
	// function call. This function displays the errorcode and string
	// description of the error. It also has an iteration input is useful when 
	// processing results in a loop (getfirst/getnext). A stack trace is also
	// displayed so that you can find the line of code where the error occurred.
	public static void ErrorHandler(int Errorcode, long Iteration, Exception excep) {
		byte[] err = new byte[255];

		if (Errorcode != LJUD.LJE_NOERROR) {
			LJUD.ErrorToString(Errorcode, err);
			System.out.println("Error number = " + Errorcode);
			System.out.println("Error string = " + new String(err));
			System.out.println("Iteration = " + Iteration);
			System.out.println("Stack Trace : ");
			excep.printStackTrace();
			if (Errorcode > LJUD.LJE_MIN_GROUP_ERROR) {
				// Quit if this is a group error.
				System.exit(1);
			}
		}
	}

	public void runExample() throws Exception
	{
		// Scanner to read input from console
		Scanner s = new Scanner(System.in);

		int intErrorcode;
		int[] intHandle={0};

		int[] intIOType={0}, intChannel={0};
		double[] dblValue={0};
		double valueAIN = 0;  //Analog Voltage Value
	
		long time=0, j=0;
		long tempChannel = 4;  //Channel which the TC/LJTIA is on (FIO4).

		double tcVolts = 0;
		double[] cjTempK = {0};
		double[] pTCTempK = {0};
		int tcType = LJUD.LJ_ttK;

		// Variables to satisfy certain method signatures
		double dummyDouble = 0;
		int dummyInt = 0;
		double[] aDummyDouble = {0};
		int[] aDummyInt = {0};

		//Set the temperature sensor to a k type thermocouple
		//Possible Thermocouple types are:
		//LJUD.LJ_ttB = 6001
		//LJUD.LJ_ttE = 6002
		//LJUD.LJ_ttJ = 6003
		//LJUD.LJ_ttK = 6004
		//LJUD.LJ_ttN = 6005
		//LJUD.LJ_ttR = 6006
		//LJUD.LJ_ttS = 6007
		//LJUD.LJ_ttT = 6008


		//Offset calibration:  The nominal voltage offset of the LJTick is
		//0.4 volts.  For improved accuracy, though, you should measure the
		//overall system offset.  We know that if the end of the TC is at the
		//same temperature as the cold junction, the voltage should be zero.
		//Put the end of the TC near the LJTIA to make sure they are at the same
		//temperature, and note the voltage measured by FIO4.  This is the actual
		//offset that can be entered below.
		double offsetVoltage = 0.4;

		//Open the first found LabJack U3 via USB.
		intErrorcode = LJUD.OpenLabJack (LJUD.LJ_dtU3, LJUD.LJ_ctUSB, "1", 1, intHandle);
		ErrorHandler(intErrorcode, 0, new Exception());


		//Reset all U3 ports to their default configuration.  
		//This is recommended prior to using the U3 to ensure the U3's ports are 
		//in the proper state.
		intErrorcode = LJUD.ePut (intHandle[0], LJUD.LJ_ioPIN_CONFIGURATION_RESET, 0, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

	
		intErrorcode = LJUD.ePut (intHandle[0], LJUD.LJ_ioPUT_ANALOG_ENABLE_BIT, tempChannel, 1, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Constantly acquire temperature readings until a key is pressed
		while(true) 
		{

			intErrorcode = 0;
			intIOType = 0;
			intChannel = 0;
			time = 0;
			tcVolts = 0;
			cjTempK = 0;
			pTCTempK = 0;
		
			//Add analog input requests.
			intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioGET_AIN, tempChannel, 0, 0, 0);
			ErrorHandler(intErrorcode, 0, new Exception());

			//Add request for internal temperature reading -- Internal temp sensor uses 
			//analog input channel 30 on the U3.
			intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioGET_AIN, 30, 0, 0, 0);
			ErrorHandler(intErrorcode, 0, new Exception());

			//Execute all requests on the labjack intHandle.
			intErrorcode = LJUD.GoOne (intHandle[0]);
			ErrorHandler(intErrorcode, 0, new Exception());

			//Get all the results.  The first result should be the voltage reading of the 
			//temperature channel.
			intErrorcode = LJUD.GetFirstResult(intHandle[0], intIOType, intChannel, valueAIN, 0, 0);
			ErrorHandler(intErrorcode, 0, new Exception());

			//The second result should be the internal temp sensor.
			intErrorcode = LJUD.GetNextResult(intHandle[0], intIOType, intChannel, cjTempK, aDummyInt, aDummyDouble);
			ErrorHandler(intErrorcode, 0, new Exception());


			//Display Voltage Reading
			System.out.println("Analog " + tempChannel + "           :" + valueAIN);

			//Display the internal temperature sensor reading.  This example uses
			//that value for cold junction compensation.
			System.out.println("U3 internal sensor :" + cjTempK[0] + " deg K");

			//To get the thermocouple voltage we subtract the offset from the AIN
			//voltage and divide by the LJTIA gain.
			tcVolts = (valueAIN - offsetVoltage)/51;

			//Convert TC voltage to temperature.
			intErrorcode = LJUD.TCVoltsToTemp(tcType, tcVolts, cjTempK[0], pTCTempK[0]);

			//Display Temperature
			System.out.println("Thermocouple sensor:" + pTCTempK + " deg K\n");

			Thread.sleep(1500);

			System.out.println("Press q to quit or press enter.\n");
			if(s.nextLine() == "q")
				break;
		}
	}
}