//---------------------------------------------------------------------------
//
//  allio.java
// 
//	Demonstrates using the add/go/get method to efficiently write and read
//	some or all analog and digital I/O on the LabJack U6.
//	Records the time for 1000 iterations and divides by 1000, to allow
//	verification of the basic command/response communication times of the
//	LabJack U6 as documented in Section 3.1 of the U6 User's Guide.
//
//  support@labjack.com
//  June 25, 2009
//----------------------------------------------------------------------
//

import java.io.*;
import java.text.*;
import com.labjack.labjackud.LJUD; // importing the LJUD class

public class U6_AllIO {

	// Loading the LJUDJava dll so that we can use the LJUDJava interface. This
	// example assumes that the dll is in the "../../../LJUDJava/" folder.
	static {
		try {
			System.loadLibrary("../../../LJUDJava/LJUDJava");
		} catch (UnsatisfiedLinkError e) {
			System.err.println("Native code library failed to load.\n" + e);
			System.exit(1);
		}
	}

	// This is our simple error handling function that is called after every UD
	// function call. This function displays the errorcode and string
	// description of the error. It also has an iteration input is useful when 
	// processing results in a loop (getfirst/getnext). A stack trace is also
	// displayed so that you can find the line of code where the error occurred.
	public static void ErrorHandler(int Errorcode, long Iteration, Exception excep) {
		byte[] err = new byte[255];

		if (Errorcode != LJUD.LJE_NOERROR) {
			LJUD.ErrorToString(Errorcode, err);
			System.out.println("Error number = " + Errorcode);
			System.out.println("Error string = " + new String(err));
			System.out.println("Iteration = " + Iteration);
			System.out.println("Stack Trace : ");
			excep.printStackTrace();
			if (Errorcode > LJUD.LJE_MIN_GROUP_ERROR) {
				// Quit if this is a group error.
				System.exit(1);
			}
		}
	}

	public void runExample() throws Exception {
		int intErrorcode;
		int[] intHandle={0};
		long lngIteration;
		char ch;

		int[] intIOType={0}, intChannel={0};
		double[] dblValue={0};
		double ValueDIPort=0;
		double[] ValueAIN = new double[16];
		ValueAIN[0] = 9999;
	

		long time=0;
		long numIterations = 100;
		int numChannels = 16;  //Number of AIN channels, 0-4.
		long resolution = 0;  //0=default, 1-8=high-speed ADC, 9-13=high-res ADC
		long settlingTime = 1;  //0=5us, 1=10us, 2=100us, 3=1ms, 4=10ms

		// Variables to satisfy certain method signatures
		double[] dummyDouble = {0};
		int[] dummyInt = {0};

		//Open the first found LabJack.
		intErrorcode = LJUD.OpenLabJack (LJUD.LJ_dtU6, LJUD.LJ_ctUSB, "1", 1, intHandle);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Configure resolution
		intErrorcode = LJUD.ePut (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chAIN_RESOLUTION, resolution, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Configure settling time
		intErrorcode = LJUD.ePut (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chAIN_SETTLING_TIME, settlingTime, 0);
		ErrorHandler(intErrorcode, 0, new Exception());
	
		//Set the timer/counter pin offset to 8, which will put the first
		//timer/counter on EIO0.
		intErrorcode = LJUD.AddRequest (intHandle[0],  LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chTIMER_COUNTER_PIN_OFFSET, 8, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Use the default clock source.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chTIMER_CLOCK_BASE, LJUD.LJ_tc48MHZ, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Enable 2 timers.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chNUMBER_TIMERS_ENABLED, 2, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Configure Timer0 as 8-bit PWM.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_MODE, 0, LJUD.LJ_tmPWM8, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Set the PWM duty cycle to 50%.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_VALUE, 0, 32768, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Configure Timer1 as 8-bit PWM.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_MODE, 1, LJUD.LJ_tmPWM8, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Set the PWM duty cycle to 50%.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_VALUE, 1, 32768, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Enable Counter0.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_COUNTER_ENABLE, 0, 1, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_COUNTER_ENABLE, 1, 1, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Execute the requests.
		intErrorcode = LJUD.GoOne (intHandle[0]);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Now add requests that will be processed every iteration of the loop.

		//Add analog input requests.
		for(int j=0;j<numChannels;j++)
		{
			intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioGET_AIN, j, 0, 0, 0);
			ErrorHandler(intErrorcode, 0, new Exception());
		}

		//Set DAC0 to 2.5 volts.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_DAC, 0, 2.5, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Read CIO digital lines.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioGET_DIGITAL_PORT, 16, 0, 4, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Only do the timer/counter stuff if there are less than 8 analog inputs.
		if(numChannels <= 8)
		{
			intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioGET_COUNTER, 0, 0, 0, 0);
			ErrorHandler(intErrorcode, 0, new Exception());

			intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioGET_COUNTER, 1, 0, 0, 0);
			ErrorHandler(intErrorcode, 0, new Exception());

			intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioGET_TIMER, 0, 0, 0, 0);
			ErrorHandler(intErrorcode, 0, new Exception());

			intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioGET_TIMER, 1, 0, 0, 0);
			ErrorHandler(intErrorcode, 0, new Exception());

			//Set the PWM duty cycle to 50%.
			intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_VALUE, 0, 32768, 0, 0);
			ErrorHandler(intErrorcode, 0, new Exception());

			//Set the PWM duty cycle to 50%.
			intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_VALUE, 1, 32768, 0, 0);
			ErrorHandler(intErrorcode, 0, new Exception());
		}


		time = System.currentTimeMillis();	

		for(int i=0;i<numIterations;i++)
		{

			//Execute the requests.
			intErrorcode = LJUD.GoOne (intHandle[0]);
			ErrorHandler(intErrorcode, i, new Exception());

			//Get all the results.  The input measurement results are stored.  All other
			//results are for configuration or output requests so we are just checking
			//whether there was an error.
			intErrorcode = LJUD.GetFirstResult(intHandle[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
			ErrorHandler(intErrorcode, i, new Exception());
			lngIteration=0;	//Used by the error handling function.
			while(intErrorcode < LJUD.LJE_MIN_GROUP_ERROR)
			{
				if (intIOType[0] == LJUD.LJ_ioGET_AIN)
					ValueAIN[intChannel[0]]=dblValue[0];

				if (intIOType[0] == LJUD.LJ_ioGET_DIGITAL_PORT)
					ValueDIPort=dblValue[0];

				intErrorcode = LJUD.GetNextResult(intHandle[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
				if(intErrorcode != LJUD.LJE_NO_MORE_DATA_AVAILABLE)
				{
					ErrorHandler(intErrorcode, lngIteration, new Exception());
				}

				lngIteration++;

			}



		}


		time = System.currentTimeMillis() - time;

		System.out.println("Milleseconds per iteration = " + (double)time / (double)numIterations);

		System.out.println("\nDigital Input = " + ValueDIPort);

		System.out.println("\nAIN readings from last iteration:\n");
		for(int j=0;j<numChannels;j++)
		{
			System.out.println(ValueAIN[j]);
		}
	}

	public static void main(String[] args) {
		try {
			new U6_AllIO().runExample();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

}