//---------------------------------------------------------------------------
//
//  SPI.cpp
//  
//	Demonstrates SPI communication.
//
//	You can short MOSI to MISO for testing.
//
//	MOSI    FIO2
//	MISO    FIO3
//	CLK     FIO0
//	CS      FIO1
//
//	If you short MISO to MOSI, then you will read back the same bytes that you write.  If you short
//	MISO to GND, then you will read back zeros.  If you short MISO to VS or leave it
//	unconnected, you will read back 255s.
//
//  support@labjack.com
//  June 26, 2009
//----------------------------------------------------------------------
//

import java.io.*;
import java.text.*;
import com.labjack.labjackud.LJUD; // importing the LJUD class

public class SPI {

	// Loading the LJUDJava dll so that we can use the LJUDJava interface. This
	// example assumes that the dll is in the "../../../LJUDJava/" folder.
	static {
		try {
			System.loadLibrary("../../../LJUDJava/LJUDJava");
		} catch (UnsatisfiedLinkError e) {
			System.err.println("Native code library failed to load.\n" + e);
			System.exit(1);
		}
	}

	// This is our simple error handling function that is called after every UD
	// function call. This function displays the errorcode and string
	// description of the error. It also has an iteration input is useful when 
	// processing results in a loop (getfirst/getnext). A stack trace is also
	// displayed so that you can find the line of code where the error occurred.
	public static void ErrorHandler(int Errorcode, int Iteration, Exception excep) {
		byte[] err = new byte[255];

		if (Errorcode != LJUD.LJE_NOERROR) {
			LJUD.ErrorToString(Errorcode, err);
			System.out.println("Error number = " + Errorcode);
			System.out.println("Error string = " + new String(err));
			System.out.println("Iteration = " + Iteration);
			System.out.println("Stack Trace : ");
			excep.printStackTrace();
			if (Errorcode > LJUD.LJE_MIN_GROUP_ERROR) {
				// Quit if this is a group error.
				System.exit(1);
			}
		}
	}

	// Java uses signed bytes but the UD driver requires unsigned bytes therefore these two methods
	// are needed to convert back and forth. For more info see http://www.lykkenborg.no/java/2005/03/how-do-i-read-unsigned-byte-in-java.html
	public static byte signed(int i) 
	{
		return (byte) i;
	}

	public static int unsigned(byte b) 
	{
		return b & 0xFF;
	}

	public void runExample() throws Exception 
	{
		int intErrorcode;
		long lngIteration;
		int[] intIOType={0}, intChannel={0};
		double[] dblValue={0};
		int[] intHandle={0};
		double[] numSPIBytesToTransfer = {0};
		byte[] dataArray = new byte[50];

		// Variables to satisfy certain method signatures
		double[] dummyDouble = {0};
		int[] dummyInt = {0};


		//Open the LabJack U6.
		intErrorcode = LJUD.OpenLabJack (LJUD.LJ_dtU6, LJUD.LJ_ctUSB, "1", 1, intHandle);
		ErrorHandler(intErrorcode, 0, new Exception());

		//First, configure the SPI communication.

		//Enable automatic chip-select control.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chSPI_AUTO_CS,1,0,0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Do not disable automatic digital i/o direction configuration.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chSPI_DISABLE_DIR_CONFIG,0,0,0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Mode A:  CPHA=1, CPOL=1.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chSPI_MODE,0,0,0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//125kHz clock.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chSPI_CLOCK_FACTOR,0,0,0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//MOSI is FIO2
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chSPI_MOSI_PIN_NUM,2,0,0);
		ErrorHandler(intErrorcode, 0, new Exception());
	
		//MISO is FIO3
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chSPI_MISO_PIN_NUM,3,0,0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//CLK is FIO0
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chSPI_CLK_PIN_NUM,0,0,0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//CS is FIO1
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chSPI_CS_PIN_NUM,1,0,0);
		ErrorHandler(intErrorcode, 0, new Exception());


		//Execute the requests on a single LabJack.  The driver will use a
		//single low-level TimerCounter command to handle all the requests above.
		intErrorcode = LJUD.GoOne(intHandle[0]);
		ErrorHandler(intErrorcode, 0, new Exception());


		//Get all the results just to check for errors.
		intErrorcode = LJUD.GetFirstResult(intHandle[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
		ErrorHandler(intErrorcode, 0, new Exception());
		lngIteration=0;	//Used by the error handling function.
		while(intErrorcode < LJUD.LJE_MIN_GROUP_ERROR)
		{
			intErrorcode = LJUD.GetNextResult(intHandle[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
			if(intErrorcode != LJUD.LJE_NO_MORE_DATA_AVAILABLE)
			{
				ErrorHandler(intErrorcode, lngIteration, new Exception());
			}

			lngIteration++;
		}


		//This example transfers 4 test bytes.
		numSPIBytesToTransfer[0] = 4;
		dataArray[0] = signed(170);
		dataArray[1] = signed(240);
		dataArray[2] = signed(170);
		dataArray[3] = signed(240);
	
		//Transfer the data.  The write and read is done at the same time.
		intErrorcode = LJUD.eGet(intHandle[0], LJUD.LJ_ioSPI_COMMUNICATION, 0, numSPIBytesToTransfer, dataArray);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Display the read data.
		System.out.println("dataArray[0] = " + unsigned(dataArray[0]));
		System.out.println("dataArray[1] = " + unsigned(dataArray[1]));
		System.out.println("dataArray[2] = " + unsigned(dataArray[2]));
		System.out.println("dataArray[3] = " + unsigned(dataArray[3]));
	}


	public static void main(String[] args) {
		try {
			new SPI().runExample();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

}