//Demonstrates using the TCVoltsToTemp function to yield temperature
//  readings for a thermocouple directly connected to a U6.
//
//  Setup:
//  TC plus to AIN0.
//  TC minus to GND.
//
//  If you do not want all the thermocouple minus leads tied together
//  at GND, you can use differential channels instead.  Connect the minus
//  to AIN1 in that case, and use a 10k resistor from AIN1 to GND to
//  provide a path for bias currents.
//
//support@labjack.com
//December 28 17, 2009


import java.io.*;
import java.text.*;
import java.util.Scanner;
import com.labjack.labjackud.LJUD; // importing the LJUD class

public class Thermocouple {

	public static void main(String[] args) 
	{
		try 
		{
			new Thermocouple().runExample();
		} 
		catch (Exception ex) 
		{
			ex.printStackTrace();
		}
	}

	// Loading the LJUDJava dll so that we can use the LJUDJava interface. This
	// example assumes that the dll is in the "../../../LJUDJava/" folder.
	static {
		try {
			System.loadLibrary("../../../LJUDJava/LJUDJava");
		} catch (UnsatisfiedLinkError e) {
			System.err.println("Native code library failed to load.\n" + e);
			System.exit(1);
		}
	}

	// This is our simple error handling function that is called after every UD
	// function call. This function displays the errorcode and string
	// description of the error. It also has an iteration input is useful when 
	// processing results in a loop (getfirst/getnext). A stack trace is also
	// displayed so that you can find the line of code where the error occurred.
	public static void ErrorHandler(int Errorcode, long Iteration, Exception excep) {
		byte[] err = new byte[255];

		if (Errorcode != LJUD.LJE_NOERROR) {
			LJUD.ErrorToString(Errorcode, err);
			System.out.println("Error number = " + Errorcode);
			System.out.println("Error string = " + new String(err));
			System.out.println("Iteration = " + Iteration);
			System.out.println("Stack Trace : ");
			excep.printStackTrace();
			if (Errorcode > LJUD.LJE_MIN_GROUP_ERROR) {
				// Quit if this is a group error.
				System.exit(1);
			}
		}
	}

	public void runExample() throws Exception
	{
		int intErrorcode;
		int[] intHandle={0};

		int[] intIOType={0}, intChannel={0};
		double[] dblValue={0};
		//double[] valueAIN = {0};  //Analog Voltage Value
		double[] ainResolution = {18};
	
		long time=0, j=0;
		int tempChannel = 0;  //Channel which the TC/LJTIA is on (AIN0).

		double[] range = {LJUD.LJ_rgBIPP1V};

		// Variables to satisfy certain method signatures
		double dummyDouble = 0;
		int dummyInt = 0;
		double[] aDummyDouble = {0};
		int[] aDummyInt = {0};
		
		double tcVolts = 0;
		double[] pTCTempK = {0};
		double[] internalTemp = {0};
		double[] cjTempK = {0};
		int tcType = LJUD.LJ_ttK;

		// A scanner to read from the console
		Scanner s = new Scanner(System.in);

		//Set the temperature sensor to a k type thermocouple
		//Possible Thermocouple types are:
		//LJUD.LJ_ttB = 6001
		//LJUD.LJ_ttE = 6002
		//LJUD.LJ_ttJ = 6003
		//LJUD.LJ_ttK = 6004
		//LJUD.LJ_ttN = 6005
		//LJUD.LJ_ttR = 6006
		//LJUD.LJ_ttS = 6007
		//LJUD.LJ_ttT = 6008


		//Offset calibration:  The nominal voltage offset of the LJTick is
		//0.4 volts.  For improved accuracy, though, you should measure the
		//overall system offset.  We know that if the end of the TC is at the
		//same temperature as the cold junction, the voltage should be zero.
		//Put the end of the TC near the LJTIA to make sure they are at the same
		//temperature, and note the voltage measured by FIO4.  This is the actual
		//offset that can be entered below.
		double offsetVoltage = 0.4;

		//Open the first found LabJack U6 via USB.
		intErrorcode = LJUD.OpenLabJack (LJUD.LJ_dtU6, LJUD.LJ_ctUSB, "1", 1, intHandle);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Configure the desired resolution.  
		intErrorcode = LJUD.eGet(intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chAIN_RESOLUTION, ainResolution, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Set the range on the analog input channel to +/-0.1 volts (x100 gain).
		intErrorcode = LJUD.eGet(intHandle[0], LJUD.LJ_ioPUT_AIN_RANGE, intChannel[0], range, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Constantly acquire temperature readings until a key is pressed
		while(true) 
		{

			intErrorcode = 0;
			intIOType[0] = 0;
			intChannel[0] = 0;
			time = 0;
			cjTempK[0] = 0;
			pTCTempK[0] = 0;
		
			//Add analog input requests.
			intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioGET_AIN, tempChannel, dummyDouble, dummyInt, dummyDouble);
			ErrorHandler(intErrorcode, 0, new Exception());

			//Add request for internal temperature reading -- Internal temp sensor uses 
			//analog input channel 14 on the U6.
			intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioGET_AIN, 14, 0, 0, 0);
			ErrorHandler(intErrorcode, 0, new Exception());

			//Execute all requests on the labjack intHandle.
			intErrorcode = LJUD.GoOne (intHandle[0]);
			ErrorHandler(intErrorcode, 0, new Exception());

			//Get all the results.  The first result should be the voltage reading of the 
			//temperature channel.
			intErrorcode = LJUD.GetFirstResult(intHandle[0], intIOType, intChannel, pTCTempK, aDummyInt, aDummyDouble);
			ErrorHandler(intErrorcode, 0, new Exception());

			//The second result should be the internal temp sensor.
			intErrorcode = LJUD.GetNextResult(intHandle[0], intIOType, intChannel, internalTemp, aDummyInt, aDummyDouble);
			ErrorHandler(intErrorcode, 0, new Exception());

			//The cold junction is the screw-terminal block where the thermocouple
			//is connected.  As discussed in the U6 User's Guide, add 2.5 degrees C
			//to the internal temp sensor reading.  If using the CB37 rather than
			//the built-in screw terminals, just add 1.0 degrees C.
			cjTempK[0] = internalTemp[0] + 2.5;

			//Display Voltage Reading
			System.out.println("Analog " + tempChannel + "           :" + pTCTempK[0]);

			//Display the internal temperature sensor reading.  This example uses
			//that value for cold junction compensation.
			System.out.println("U6 internal sensor :" + internalTemp[0] + " deg K");

			//Convert TC voltage to temperature.
			intErrorcode = LJUD.TCVoltsToTemp(tcType, pTCTempK[0], cjTempK[0], pTCTempK);

			//Display Temperature
			System.out.println("Thermocouple sensor: " + pTCTempK[0] + " deg K");

			System.out.println("Press q to quit or enter to go again!\n");

			if(s.nextLine().equals("q"))
				break;
		}
	}
}