//---------------------------------------------------------------------------
//
//  TimerCounter.java
//  
//  Basic U6 example does a PWM output and a counter input, using AddGoGet method.
//
//  support@labjack.com
//  June 29, 2009
//----------------------------------------------------------------------

import java.io.*;
import java.text.*;
import java.util.Scanner;
import com.labjack.labjackud.LJUD; // importing the LJUD class

public class TimerCounter {

	public static void main(String[] args) 
	{
		try 
		{
			new TimerCounter().runExample();
		} 
		catch (Exception ex) 
		{
			ex.printStackTrace();
		}
	}

	// Loading the LJUDJava dll so that we can use the LJUDJava interface. This
	// example assumes that the dll is in the "../../../LJUDJava/" folder.
	static {
		try {
			System.loadLibrary("../../../LJUDJava/LJUDJava");
		} catch (UnsatisfiedLinkError e) {
			System.err.println("Native code library failed to load.\n" + e);
			System.exit(1);
		}
	}

	// This is our simple error handling function that is called after every UD
	// function call. This function displays the errorcode and string
	// description of the error. It also has an iteration input is useful when 
	// processing results in a loop (getfirst/getnext). A stack trace is also
	// displayed so that you can find the line of code where the error occurred.
	public static void ErrorHandler(int Errorcode, long Iteration, Exception excep) {
		byte[] err = new byte[255];

		if (Errorcode != LJUD.LJE_NOERROR) {
			LJUD.ErrorToString(Errorcode, err);
			System.out.println("Error number = " + Errorcode);
			System.out.println("Error string = " + new String(err));
			System.out.println("Iteration = " + Iteration);
			System.out.println("Stack Trace : ");
			excep.printStackTrace();
			if (Errorcode > LJUD.LJE_MIN_GROUP_ERROR) {
				// Quit if this is a group error.
				System.exit(1);
			}
		}
	}

	public void runExample() throws Exception
	{
		int intErrorcode;
		long lngGetNextIteration;
		int[] intIOType={0}, intChannel={0};
		double[] dblValue={0};
		int[] intHandle={0};

		// Variables to satisfy certain method signatures
		double[] dummyDouble = {0};
		int[] dummyInt = {0};

		//Open the first found LabJack U6.
		intErrorcode = LJUD.OpenLabJack (LJUD.LJ_dtU6, LJUD.LJ_ctUSB, "1", 1, intHandle);
		ErrorHandler(intErrorcode, 0, new Exception());


		//First requests to configure the timer and counter.  These will be
		//done with and add/go/get block.

		//Set the timer/counter pin offset to 0, which will put the first
		//timer/counter on FIO0.
		intErrorcode = LJUD.AddRequest (intHandle[0],  LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chTIMER_COUNTER_PIN_OFFSET, 0, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Use the 48 MHz timer clock base with divider.  Since we are using clock with divisor
		//support, Counter0 is not available.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chTIMER_CLOCK_BASE, LJUD.LJ_tc48MHZ_DIV, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Set the divisor to 48 so the actual timer clock is 1 MHz.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chTIMER_CLOCK_DIVISOR, 48, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Enable 1 timer.  It will use FIO0.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chNUMBER_TIMERS_ENABLED, 1, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Configure Timer0 as 8-bit PWM.  Frequency will be 1M/256 = 3906 Hz.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_MODE, 0, LJUD.LJ_tmPWM8, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Set the PWM duty cycle to 50%.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_VALUE, 0, 32768, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Enable Counter1.  It will use FIO1 since 1 timer is enabled.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_COUNTER_ENABLE, 1, 1, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Execute the requests.
		intErrorcode = LJUD.GoOne (intHandle[0]);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Get all the results just to check for errors.
		intErrorcode = LJUD.GetFirstResult(intHandle[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
		ErrorHandler(intErrorcode, 0, new Exception());
		lngGetNextIteration=0;	//Used by the error handling function.
		while(intErrorcode < LJUD.LJE_MIN_GROUP_ERROR)
		{
			intErrorcode = LJUD.GetNextResult(intHandle[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
			if(intErrorcode != LJUD.LJE_NO_MORE_DATA_AVAILABLE)
			{
				ErrorHandler(intErrorcode, lngGetNextIteration, new Exception());
			}

			lngGetNextIteration++;
		}

		//Wait 1 second.
		Thread.sleep(1000);

		//Request a read from the counter.
		intErrorcode = LJUD.eGet(intHandle[0], LJUD.LJ_ioGET_COUNTER, 1, dblValue, 0);
		ErrorHandler(intErrorcode, 0, new Exception());
  
		//This should read roughly 4k counts if FIO0 is shorted to FIO1.
		System.out.println("Counter = " + dblValue[0]);

		//Wait 1 second.
		Thread.sleep(1000);

		//Request a read from the counter.
		intErrorcode = LJUD.eGet(intHandle[0], LJUD.LJ_ioGET_COUNTER, 1, dblValue, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//This should read about 3906 counts more than the previous read.
		System.out.println("Counter = " + dblValue[0]);
	

		//Reset all pin assignments to factory default condition.
		intErrorcode = LJUD.ePut (intHandle[0], LJUD.LJ_ioPIN_CONFIGURATION_RESET, 0, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//The PWM output sets FIO0 to output, so we do a read here to set
		//it to input.
		intErrorcode = LJUD.eGet (intHandle[0], LJUD.LJ_ioGET_DIGITAL_BIT, 0, dblValue, 0);
		ErrorHandler(intErrorcode, 0, new Exception());
		}
	}