//---------------------------------------------------------------------------
//
//  AllIO.java
// 
//	Demonstrates using the add/go/get method to efficiently write and read
//	virtually all analog and digital I/O on the LabJack UE9.
//	Records the time for 1000 iterations and divides by 1000, to allow
//	verification of the basic command/response communication times of the
//	LabJack UE9 as documented in Section 3.1 of the UE9 User's Guide.
//
//  support@labjack.com
//  June 25, 2009
//----------------------------------------------------------------------
//

import java.io.*;
import java.text.*;
import com.labjack.labjackud.LJUD;     //importing the LJUD class

public class AllIO {
    
    //Loading the LJUDJava dll so that we can use the LJUDJava JNI.  This
    //example assumes that the dll is in the "../../../LJUDJava/" folder.
    static {
        try {
            System.loadLibrary("../../../LJUDJava/LJUDJava");
        } catch (UnsatisfiedLinkError e) {
            System.err.println("Native code library failed to load.\n" + e);
            System.exit(1);
        }
    }
    
    //This is our simple error handling function that is called after every UD
    //function call.  This function displays the errorcode and string description
    //of the error.  It also has an iteration input is useful when processing
    //results in a loop (getfirst/getnext).  A stack trace is also displayed so
    //that you can find the line of code where the error occurred.
    public static void ErrorHandler(int Errorcode, long Iteration, Exception excep) {
        byte []err = new byte[255];
        
        if (Errorcode != LJUD.LJE_NOERROR) {
            LJUD.ErrorToString(Errorcode, err);
            System.out.println("Error number = " + Errorcode);
            System.out.println("Error string = " + new String(err));
            System.out.println("Iteration = " + Iteration);
            System.out.println("Stack Trace : ");
            excep.printStackTrace();
            if(Errorcode > LJUD.LJE_MIN_GROUP_ERROR) {
                //Quit if this is a group error.
                System.exit(1);
            }
        }
    }
    
    public void runExample() throws Exception {
        
		int intErrorcode;
		int[] intHandle={0};
		char ch;
		long lngIteration;

		int[] intIOType={0}, intChannel={0};
		double[] dblValue={0};
		double ValueDIPort=0;
		double[] ValueAIN = new double[16];
		ValueAIN[0] = 9999;
	

		long time=0;
		long numIterations = 1000;
		int numChannels = 5;  //Number of AIN channels, 0-16.
		long ainResolution = 12;

		// Variables to satisfy certain method signatures
		int[] dummyInt = {0};
		double[] dummyDouble = {0};

		//Open the first found LabJack UE9.
		intErrorcode = LJUD.OpenLabJack (LJUD.LJ_dtUE9, LJUD.LJ_ctUSB, "1", 1, intHandle);
		//intErrorcode = LJUD.OpenLabJack (LJUD.LJ_dtUE9, LJUD.LJ_ctETHERNET, "192.168.1.38", 0, intHandle);
		ErrorHandler(intErrorcode, 0, new Exception());


		//Set DAC0 to 2.5 volts.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_DAC, 0, 2.5, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Set DAC1 to 3.5 volts.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_DAC, 1, 3.5, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Write all digital I/O.  Doing a bunch of bit instructions, rather than
		//the following port instruction, should not make a noticable difference
		//in the overall execution time.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_DIGITAL_PORT, 0, 0, 23, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Configure the desired resolution.  Note that depending on resolution and
		//number of analog inputs, numIterations might need to be reduced from the
		//default above so the program does not take too long to execute.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chAIN_RESOLUTION, ainResolution, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Add analog input requests.
		for(int j=0;j<numChannels;j++)
		{
			intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioGET_AIN, j, 0, 0, 0);
			ErrorHandler(intErrorcode, 0, new Exception());
		}

		//Request a read of all digital I/O.  Doing a bunch of bit instructions,
		//rather than the following port instruction, should not make a noticable
		//difference in the overall execution time.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioGET_DIGITAL_PORT, 0, 0, 23, 0);
		ErrorHandler(intErrorcode, 0, new Exception());


		time = System.currentTimeMillis();

		for(int i=0;i<numIterations;i++)
		{

			//Execute the requests.
			intErrorcode = LJUD.GoOne (intHandle[0]);
			ErrorHandler(intErrorcode, i, new Exception());

			//Get all the results.  The input measurement results are stored.  All other
			//results are for configuration or output requests so we are just checking
			//whether there was an error.
			intErrorcode = LJUD.GetFirstResult(intHandle[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
			ErrorHandler(intErrorcode, i, new Exception());
			lngIteration=0;	//Used by the error handling function.
			while(intErrorcode < LJUD.LJE_MIN_GROUP_ERROR)
			{
				if (intIOType[0] == LJUD.LJ_ioGET_AIN)
					ValueAIN[intChannel[0]]=dblValue[0];
				else if (intIOType[0] == LJUD.LJ_ioGET_DIGITAL_PORT)
					ValueDIPort=dblValue[0];

				intErrorcode = LJUD.GetNextResult(intHandle[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
				if(intErrorcode != LJUD.LJE_NO_MORE_DATA_AVAILABLE)
				{
					ErrorHandler(intErrorcode, lngIteration, new Exception());
				}

				lngIteration++;

			}

		}


		time = System.currentTimeMillis() - time;

		System.out.println("Milleseconds per iteration = " + (double)time / (double)numIterations);

		System.out.println("\nDigital Input = " + ValueDIPort);

		System.out.println("\nAIN readings from last iteration:\n");
		System.out.println(numChannels);
		for(int j=0;j<numChannels;j++)
		{
			System.out.println(ValueAIN[j]);
		}
    }
    
    public static void main(String[] args) {
        try {
            new AllIO().runExample();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}

