/*
 *  UE9eFunctions.java
 *  
 *  Demonstrates the UD E-functions with the LabJack UE9 with the 
 *  LJUDJava JNI wrapper.
 *
 *  support@labjack.com
 *  July 26, 2007
 */

import java.io.*;
import java.text.*;
import com.labjack.labjackud.LJUD;     //importing the LJUD class

public class UE9eFunctions {
    
    //Loading the LJUDJava dll so that we can use the LJUDJava interface.  This
    //example assumes that the dll is in the "../../../LJUDJava/" folder.
    static {
        try {
            System.loadLibrary("../../../LJUDJava/LJUDJava");
        } catch (UnsatisfiedLinkError e) {
            System.err.println("Native code library failed to load.\n" + e);
            System.exit(1);
        }
    }
    
    //This is our simple error handling function that is called after every UD
    //function call.  This function displays the errorcode and string description
    //of the error.  It also has an iteration input is useful when processing
    //results in a loop (getfirst/getnext).  A stack trace is also displayed so
    //that you can find the line of code where the error occurred.
    public static void ErrorHandler(int Errorcode, int Iteration, Exception excep) {
        byte []err = new byte[255];
        
        if (Errorcode != LJUD.LJE_NOERROR) {
            LJUD.ErrorToString(Errorcode, err);
            System.out.println("Error number = " + Errorcode);
            System.out.println("Error string = " + new String(err));
            System.out.println("Iteration = " + Iteration);
            System.out.println("Stack Trace : ");
            excep.printStackTrace();
            if(Errorcode > LJUD.LJE_MIN_GROUP_ERROR) {
                //Quit if this is a group error.
                System.exit(1);
            }
        }
    }
    
    public void runExample() throws Exception {

	int errorcode;
	double []dblValue = {0};
	int []value = {0};
	int []handle = {0};

	int range, resolution, binary;
	int []aEnableTimers = new int[6];
	int []aEnableCounters = new int[2];
	int timerClockBaseIndex, timerClockDivisor;
	int []aTimerModes = new int[6];
	double []adblTimerValues = new double[6];
	int []aReadTimers = new int[6];
	int []aUpdateResetTimers = new int[6];
	int []aReadCounters = new int[2];
	int []aResetCounters = new int[2];
	double []adblCounterValues = new double[2];
	double highTime, lowTime, dutyCycle;

	NumberFormat formatter = new DecimalFormat("0.000");
        
	//Read and display the UD and LJUDJava versions.
	System.out.println("UD Driver Version = " + formatter.format(LJUD.GetDriverVersion()));
        System.out.println("LJUD JNI Version = " + formatter.format(LJUD.LJUDJAVA_VERSION) + "\n");

	//Open the first found LabJack UE9.
	errorcode = LJUD.OpenLabJack (LJUD.LJ_dtUE9, LJUD.LJ_ctUSB, "1", 1, handle);
	//errorcode = LJUD.OpenLabJack (LJUD.LJ_dtUE9, LJUD.LJ_ctETHERNET, "192.168.1.209", 0, handle);
	ErrorHandler(errorcode, 0, new Exception());

	//Start by using the pin_configuration_reset IOType so that all
	//pin assignments are in the factory default condition.
	errorcode = LJUD.ePut (handle[0], LJUD.LJ_ioPIN_CONFIGURATION_RESET, 0, 0, 0);
	ErrorHandler(errorcode, 0, new Exception());

	//Take a measurement from AIN3.
	range = LJUD.LJ_rgBIP5V;
	resolution = 17;
	binary = 0;
	errorcode = LJUD.eAIN (handle[0], 3, 0, dblValue, range, resolution, -1, binary, 0, 0);
	ErrorHandler(errorcode, 0, new Exception());
	System.out.println("AIN3 = " + dblValue[0]);

	//Set DAC0 to 3.0 volts.
	dblValue[0] = 3.0;
	binary = 0;
	errorcode = LJUD.eDAC (handle[0], 0, dblValue[0], binary, 0, 0);
	ErrorHandler(errorcode, 0, new Exception());
	System.out.println("DAC0 set to " + dblValue[0] + " volts");

	//Read state of FIO2.
	errorcode = LJUD.eDI (handle[0], 2, value);
	ErrorHandler(errorcode, 0, new Exception());
	System.out.println("FIO2 = " + value[0]);

	//Set the state of FIO3.
	value[0] = 0;
	errorcode = LJUD.eDO (handle[0], 3, value[0]);
	ErrorHandler(errorcode, 0, new Exception());
	System.out.println("FIO3 set to = " + value[0] + "\n");

	//Timers and Counters example.
	//First, a call to eTCConfig.  Fill the arrays with the desired values, then make the call.
	timerClockBaseIndex = LJUD.LJ_tc750KHZ;  //Choose 750 kHz base clock.
	timerClockDivisor = 3; //Divide by 3, thus timer clock is 250 kHz.
	aEnableTimers[0] = 1; //Enable Timer0 (uses FIO0).
	aEnableTimers[1] = 1; //Enable Timer1 (uses FIO1).
	aEnableTimers[2] = 1; //Enable Timer2 (uses FIO2).
	aEnableTimers[3] = 1; //Enable Timer3 (uses FIO3).
	aEnableTimers[4] = 0; //Disable Timer4.
	aEnableTimers[5] = 0; //Disable Timer5.
	aTimerModes[0] = LJUD.LJ_tmPWM8; //Timer0 is 8-bit PWM output.  Frequency is 250k/256 = 977 Hz.
	aTimerModes[1] = LJUD.LJ_tmDUTYCYCLE; //Timer1 is duty cyle input.
	aTimerModes[2] = LJUD.LJ_tmFIRMCOUNTER; //Timer2 is firmware counter input.
	aTimerModes[3] = LJUD.LJ_tmRISINGEDGES16; //Timer3 is 16-bit period measurement.
	aTimerModes[4] = 0; //Timer4 not enabled.
	aTimerModes[5] = 0; //Timer5 not enabled.
	adblTimerValues[0] = 16384; //Set PWM8 duty-cycle to 75%.
	adblTimerValues[1] = 0;
	adblTimerValues[2] = 0;
	adblTimerValues[3] = 0;
	adblTimerValues[4] = 0;
	adblTimerValues[5] = 0;
	aEnableCounters[0] = 1; //Enable Counter0 (uses FIO4).
	aEnableCounters[1] = 1; //Enable Counter1 (uses FIO5).
	errorcode = LJUD.eTCConfig(handle[0], aEnableTimers, aEnableCounters, 0, timerClockBaseIndex, timerClockDivisor, aTimerModes, adblTimerValues, 0, 0);
	ErrorHandler(errorcode, 0, new Exception());
	System.out.println("Timers and Counters enabled.\n");

	Thread.sleep(1000); //Wait 1 second.

	//Now, a call to eTCValues.
	aReadTimers[0] = 0; //Don't read Timer0 (output timer).
	aReadTimers[1] = 1; //Read Timer1;
	aReadTimers[2] = 1; //Read Timer2;
	aReadTimers[3] = 1; //Read Timer3;
	aReadTimers[4] = 0; //Timer4 not enabled.
	aReadTimers[5] = 0; //Timer5 not enabled.
	aUpdateResetTimers[0] = 1; //Update Timer0;
	aUpdateResetTimers[1] = 1; //Reset Timer1;
	aUpdateResetTimers[2] = 1; //Reset Timer2;
	aUpdateResetTimers[3] = 1; //Reset Timer3;
	aUpdateResetTimers[4] = 0; //Timer4 not enabled.
	aUpdateResetTimers[5] = 0; //Timer5 not enabled.
	aReadCounters[0] = 1; //Read Counter0;
	aReadCounters[1] = 1; //Read Counter1;
	aResetCounters[0] = 1; //Reset Counter0.
	aResetCounters[1] = 1; //Reset Counter1.
	adblTimerValues[0] = 32768; //Change Timer0 duty-cycle to 50%.
	adblTimerValues[1] = 0;
	adblTimerValues[2] = 0;
	adblTimerValues[3] = 0;
	adblTimerValues[4] = 0;
	adblTimerValues[5] = 0;
	errorcode = LJUD.eTCValues(handle[0], aReadTimers, aUpdateResetTimers, aReadCounters, aResetCounters, adblTimerValues, adblCounterValues, 0, 0);
	ErrorHandler(errorcode, 0, new Exception());
	System.out.println("Timer1 value = " + formatter.format(adblTimerValues[1]));
	System.out.println("Timer2 value = " + formatter.format(adblTimerValues[2]));
	System.out.println("Timer3 value = " + formatter.format(adblTimerValues[3]));
	System.out.println("Counter0 value = " + formatter.format(adblCounterValues[0]));
	System.out.println("Counter1 value = " + formatter.format(adblCounterValues[1]));


	//Convert Timer1 value to duty-cycle percentage.
	//High time is LSW
	highTime = (double)(((long)adblTimerValues[1]) % (65536));
	//Low time is MSW
	lowTime = (double)(((long)adblTimerValues[1]) / (65536));
	//Calculate the duty cycle percentage.
	dutyCycle = (( (highTime+lowTime) != 0)?(100*highTime/(highTime+lowTime)):0);
	formatter = new DecimalFormat("0.0");
	System.out.println("\nHigh clicks Timer1 = " + formatter.format(highTime));
	System.out.println("Low clicks Timer1 = " + formatter.format(lowTime));
	System.out.println("Duty cycle Timer1 = " + formatter.format(dutyCycle));


	//Disable all timers and counters.
	aEnableTimers[0] = 0;
	aEnableTimers[1] = 0;
	aEnableTimers[2] = 0;
	aEnableTimers[3] = 0;
	aEnableTimers[4] = 0;
	aEnableTimers[5] = 0;
	aEnableCounters[0] = 0;
	aEnableCounters[1] = 0;
	errorcode = LJUD.eTCConfig(handle[0], aEnableTimers, aEnableCounters, 0, timerClockBaseIndex, timerClockDivisor, aTimerModes, adblTimerValues, 0, 0);
	ErrorHandler(errorcode, 0, new Exception());
    }
    
    public UE9eFunctions() {}
    
    public static void main(String[] args) {
        try {
            new UE9eFunctions().runExample();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
}