//---------------------------------------------------------------------------
//
//  ue9_externaltriggerstream.java
//  
//  Does a 6-channel externally triggered stream of AIN0, AIN1, EIO_FIO,
//	MIO_CIO, Timer0_Low, and Timer0_High.
//
//	Timer0 (FIO0) is enabled as SystemTimerLow (mode 10).  With an externally
//	triggered stream, the timing is determined by the external trigger signal,
//	so this timer value can be used to timestamp each scan.
//
//	Timer1 (FIO1) is enabled as frequency output at 1 kHz, and can be
//	used as the external trigger signal for testing.
//
//	External triggering is enabled for the stream, so the trigger signal should
//	be connected to FIO2.
//
//	For testing, connect FIO1 to FIO2.
//
//  support@labjack.com
//  June 29, 2009
//----------------------------------------------------------------------
//

import java.io.*;
import java.text.*;
import java.util.Scanner;
import com.labjack.labjackud.LJUD;     //importing the LJUD class

public class ExternalTriggerStream {
    
    //Loading the LJUDJava dll so that we can use the LJUDJava JNI.  This
    //example assumes that the dll is in the "../../../LJUDJava/" folder.
    static {
        try {
            System.loadLibrary("../../../LJUDJava/LJUDJava");
        } catch (UnsatisfiedLinkError e) {
            System.err.println("Native code library failed to load.\n" + e);
            System.exit(1);
        }
    }
    
    //This is our simple error handling function that is called after every UD
    //function call.  This function displays the errorcode and string description
    //of the error.  It also has an iteration input is useful when processing
    //results in a loop (getfirst/getnext).  A stack trace is also displayed so
    //that you can find the line of code where the error occurred.
    public static void ErrorHandler(int Errorcode, int Iteration, Exception excep) {
        byte []err = new byte[255];
        
        if (Errorcode != LJUD.LJE_NOERROR) {
            LJUD.ErrorToString(Errorcode, err);
            System.out.println("Error number = " + Errorcode);
            System.out.println("Error string = " + new String(err));
            System.out.println("Iteration = " + Iteration);
            System.out.println("Stack Trace : ");
            excep.printStackTrace();
            if(Errorcode > LJUD.LJE_MIN_GROUP_ERROR) {
                //Quit if this is a group error.
                System.exit(1);
            }
        }
    }
    
    public void runExample() throws Exception {
        
		int intErrorcode;
		int[] intHandle={0};
		int iteration;
		int[] intIOType={0}, intChannel={0};
		double[] dblValue={0};
		double[] dblCommBacklog = {0};
		boolean finished = false;
		Scanner s = new Scanner(System.in);
		int i = 0;

		// Variables to satisfy certain method signatures
		double[] dummyDouble = {0};
		int[] dummyInt = {0};

		//The actual scan rate is determined by the external clock, but we need
		//an idea of how fast the scan rate will be so that we can make
		//the buffers big enough.  Also, the driver needs to have an idea of the
		//expected scan rate to help it decide how big of packets to transfer.
		double scanRate = 1000;
		long delayms = 1000;
		double numScans = 2000;  //2x the expected # of scans (2*scanRate*delayms/1000)
		double[] numScansRequested = new double[1];
		double[] adblData = new double[12000];  //Max buffer size (#channels*numScansRequested)

		//Open the first found LabJack UE9 over USB.
		intErrorcode = LJUD.OpenLabJack (LJUD.LJ_dtUE9, LJUD.LJ_ctUSB, "1", 1, intHandle);
		//intErrorcode = LJUD.OpenLabJack (LJUD.LJ_dtUE9, LJUD.LJ_ctETHERNET, "192.168.1.184", 0, intHandle);
		ErrorHandler(intErrorcode, 0, new Exception());


		//Make sure the UE9 is not streaming.
		intErrorcode = LJUD.eGet(intHandle[0], LJUD.LJ_ioSTOP_STREAM, 0, dummyDouble, 0);
		ErrorHandler(intErrorcode, 0, new Exception());


		//Disable all timers and counters to put everything in a known initial state.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chNUMBER_TIMERS_ENABLED, 0, 0, 0);
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_COUNTER_ENABLE, 0, 0, 0, 0);
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_COUNTER_ENABLE, 1, 0, 0, 0);
		intErrorcode = LJUD.GoOne(intHandle[0]);


		//First we will configure Timer0 as system timer low and configure Timer1 to
		//output a 1000 Hz square wave.

		//Use the fixed 750kHz timer clock source.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chTIMER_CLOCK_BASE, LJUD.LJ_tc750KHZ, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Set the divisor to 3 so the actual timer clock is 250 kHz.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chTIMER_CLOCK_DIVISOR, 3, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Enable 2 timers.  They will use FIO0-FIO1.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chNUMBER_TIMERS_ENABLED, 2, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Configure Timer0 as system timer low.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_MODE, 0, LJUD.LJ_tmSYSTIMERLOW, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Configure Timer1 as frequency output.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_MODE, 1, LJUD.LJ_tmFREQOUT, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Set the frequency output on Timer1 to 1000 Hz (250000/(2*125) = 1000).
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_VALUE, 1, 125, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Execute the requests on a single LabJack.  The driver will use a
		//single low-level TimerCounter command to handle all the requests above.
		intErrorcode = LJUD.GoOne(intHandle[0]);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Get all the results just to check for errors.
		intErrorcode = LJUD.GetFirstResult(intHandle[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
		ErrorHandler(intErrorcode, 0, new Exception());
		iteration=0;	//Used by the error handling function.
		while(intErrorcode < LJUD.LJE_MIN_GROUP_ERROR)
		{
			intErrorcode = LJUD.GetNextResult(intHandle[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
			if(intErrorcode != LJUD.LJE_NO_MORE_DATA_AVAILABLE)
			{
				ErrorHandler(intErrorcode, iteration, new Exception());
			}

			iteration++;
		}




		//Configure the stream:
		//Configure resolution for all analog inputs.  Since the test external clock
		//is at 1000 Hz, and we are scanning 6 channels, we will have a
		//sample rate of 6000 samples/second.  That means the maximum resolution
		//we could use is 13-bit.  We will use 12-bit in this example.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chAIN_RESOLUTION, 12, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());
		//Configure the analog input range on channel 0 for bipolar +-5 volts.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_AIN_RANGE, 0, LJUD.LJ_rgBIP5V, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());
		//Configure the analog input range on channel 1 for bipolar +-5 volts.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_AIN_RANGE, 1, LJUD.LJ_rgBIP5V, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());
		//Give the driver a 5 second buffer (scanRate * 6 channels * 5 seconds).
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chSTREAM_BUFFER_SIZE, scanRate*6*5, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());
		//Configure reads to retrieve whatever data is available without waiting (wait mode LJUD.LJ_swNONE).
		//See comments below to change this program to use LJUD.LJ_swSLEEP mode.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chSTREAM_WAIT_MODE, LJUD.LJ_swNONE, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());
		//Configure for external triggering.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chSTREAM_EXTERNAL_TRIGGER, 1, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());
		//Define the scan list.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioCLEAR_STREAM_CHANNELS, 0, 0, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioADD_STREAM_CHANNEL, 0, 0, 0, 0);	//AIN0
		ErrorHandler(intErrorcode, 0, new Exception());
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioADD_STREAM_CHANNEL, 1, 0, 0, 0);	//AIN1
		ErrorHandler(intErrorcode, 0, new Exception());
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioADD_STREAM_CHANNEL, 193, 0, 0, 0);	//EIO_FIO
		ErrorHandler(intErrorcode, 0, new Exception());
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioADD_STREAM_CHANNEL, 194, 0, 0, 0);	//MIO_CIO
		ErrorHandler(intErrorcode, 0, new Exception());
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioADD_STREAM_CHANNEL, 200, 0, 0, 0);	//Timer0 LSW
		ErrorHandler(intErrorcode, 0, new Exception());
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioADD_STREAM_CHANNEL, 224, 0, 0, 0);	//Timer0 MSW
		ErrorHandler(intErrorcode, 0, new Exception());
    
		//Execute the list of requests.
		intErrorcode = LJUD.GoOne(intHandle[0]);
		ErrorHandler(intErrorcode, 0, new Exception());
    
		//Get all the results just to check for errors.
		intErrorcode = LJUD.GetFirstResult(intHandle[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
		ErrorHandler(intErrorcode, 0, new Exception());
		iteration=0;	//Used by the error handling function.
		while(intErrorcode < LJUD.LJE_MIN_GROUP_ERROR)
		{
			intErrorcode = LJUD.GetNextResult(intHandle[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
			if(intErrorcode != LJUD.LJE_NO_MORE_DATA_AVAILABLE)
			{
				ErrorHandler(intErrorcode, iteration, new Exception());
			}

			iteration++;
		}
    
		//Start the stream.
		intErrorcode = LJUD.eGet(intHandle[0], LJUD.LJ_ioSTART_STREAM, 0, dblValue, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

  

		//Read data
		while(!finished)
		{
			//Since we are using wait mode LJUD.LJ_swNONE, we will wait a little, then
			//read however much data is available.  Thus this delay will control how
			//fast the program loops and how much data is read each loop.  An
			//alternative common method is to use wait mode LJUD.LJ_swSLEEP where the
			//stream read waits for a certain number of scans.  In such a case
			//you would not have a delay here, since the stream read will actually
			//control how fast the program loops.
			//
			//To change this program to use sleep mode,
			//	-change numScans to the actual number of scans desired per read,
			//	-change wait mode addrequest value to LJUD.LJ_swSLEEP,
			//	-comment out the following Sleep command.

			Thread.sleep(delayms);	//Remove if using LJUD.LJ_swSLEEP.

			//init array so we can easily tell if it has changed
			for(int k=0;k<numScans*2;k++)
			{
				adblData[k] = 9999.0;
			}

			//Read the data.  We will request twice the number we expect, to
			//make sure we get everything that is available.
			//Note that the array we pass must be sized to hold enough SAMPLES, and
			//the Value we pass specifies the number of SCANS to read.
			numScansRequested[0]=numScans;
			intErrorcode = LJUD.eGet(intHandle[0], LJUD.LJ_ioGET_STREAM_DATA, LJUD.LJ_chALL_CHANNELS, numScansRequested, adblData);
			//This displays the number of scans that were actually read.
			System.out.println("\nIteration #" + i);
			System.out.println("Number read = " + numScansRequested[0]);
			//This displays just the first scan.
			System.out.println("First scan = "  + adblData[0] + ", " + adblData[1] + ", " + adblData[2] + ", " + adblData[3] + ", " + adblData[4] + ", " + adblData[5]);
			ErrorHandler(intErrorcode, 0, new Exception());
			//Retrieve the current Comm backlog.  The UD driver retrieves stream data from
			//the UE9 in the background, but if the computer is too slow for some reason
			//the driver might not be able to read the data as fast as the UE9 is
			//acquiring it, and thus there will be data left over in the UE9 buffer.
			intErrorcode = LJUD.eGet(intHandle[0], LJUD.LJ_ioGET_CONFIG, LJUD.LJ_chSTREAM_BACKLOG_COMM, dblCommBacklog, 0);
			System.out.println("Comm Backlog = " + dblCommBacklog[0]);
			System.out.println("\nPress enter to go again or press q to finish");
			finished = s.nextLine().equals("q");
			i++;
		}

   
		//Stop the stream
		intErrorcode = LJUD.eGet(intHandle[0], LJUD.LJ_ioSTOP_STREAM, 0, dummyDouble, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Disable the timers.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chNUMBER_TIMERS_ENABLED, 0, 0, 0);
		intErrorcode = LJUD.GoOne(intHandle[0]);

    }
    
    public static void main(String[] args) {
        try {
            new ExternalTriggerStream().runExample();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}

