//---------------------------------------------------------------------------
//
//  ue9_memory.c
//  
//  Demonstrates writing and reading to the user and cal memory on the UE9.
//
//  support@labjack.com
//  June 29, 2009
//---------------------------------------------------------------------------
//

import java.io.*;
import java.util.Random;
import java.text.*;
import java.util.Scanner;
import com.labjack.labjackud.LJUD; // importing the LJUD class

public class Memory {

	// Loading the LJUDJava dll so that we can use the LJUDJava interface. This
	// example assumes that the dll is in the "../../../LJUDJava/" folder.
	static {
		try {
			System.loadLibrary("../../../LJUDJava/LJUDJava");
		} catch (UnsatisfiedLinkError e) {
			System.err.println("Native code library failed to load.\n" + e);
			System.exit(1);
		}
	}

	// This is our simple error handling function that is called after every UD
	// function call. This function displays the errorcode and string
	// description of the error. It also has an iteration input is useful when 
	// processing results in a loop (getfirst/getnext). A stack trace is also
	// displayed so that you can find the line of code where the error occurred.
	public static void ErrorHandler(int Errorcode, long Iteration, Exception excep) {
		byte[] err = new byte[255];

		if (Errorcode != LJUD.LJE_NOERROR) {
			LJUD.ErrorToString(Errorcode, err);
			System.out.println("Error number = " + Errorcode);
			System.out.println("Error string = " + new String(err));
			System.out.println("Iteration = " + Iteration);
			System.out.println("Stack Trace : ");
			excep.printStackTrace();
			if (Errorcode > LJUD.LJE_MIN_GROUP_ERROR) {
				// Quit if this is a group error.
				System.exit(1);
			}
		}
	}

	// Java uses signed bytes but the UD driver requires unsigned bytes therefore these two methods
	// are needed to convert back and forth. For more info see http://www.lykkenborg.no/java/2005/03/how-do-i-read-unsigned-byte-in-java.html
	public static byte signed(int i) 
	{
		return (byte) i;
	}

	public static int unsigned(byte b) 
	{
		return b & 0xFF;
	}

	public void runExample() throws Exception 
	{
		int intErrorcode;
		int[] intHandle={0};
		int i=0;
		byte[] achrUserMem =  new byte[1024];
		double[] adblCalMem = new double[1024];

		// Variables that satisfy certain method signatures
		int[] dummyInt = {0};
		double[] dummyDouble = {0};

		//Seed the random number function.
		Random rand = new Random(System.currentTimeMillis());

		//Open the first found LabJack UE9.
		intErrorcode = LJUD.OpenLabJack (LJUD.LJ_dtUE9, LJUD.LJ_ctUSB, "1", 1, intHandle);
		//intErrorcode = LJUD.OpenLabJack (LJUD.LJ_dtUE9, LJUD.LJ_ctETHERNET, "192.168.1.209", 0, intHandle);
		ErrorHandler(intErrorcode, 0, new Exception());

		//First a user memory example.  We will read the memory, update a few elements,
		//and write the memory.  The entire memory area is read and written each time.
		//The user memory is just stored as bytes, so almost any information can be
		//put in there such as integers, doubles, or strings.

		//Read the user memory.
		intErrorcode = LJUD.eGet(intHandle[0], LJUD.LJ_ioGET_CONFIG, LJUD.LJ_chUSER_MEM, dummyDouble, achrUserMem);
		ErrorHandler(intErrorcode, 0, new Exception());
		//Display the first 4 elements.
		System.out.println("Read User Mem [0-3] = " + achrUserMem[0] + ", " + achrUserMem[1] + ", " + achrUserMem[2] + ", " + achrUserMem[3]);
		//Create 4 new pseudo-random numbers to write.  We will update the first
		//4 elements of user memory, but the rest will be unchanged.
		for(i=0;i<4;i++)
		{
			achrUserMem[i] = (byte)(255*((float)rand.nextDouble()/1.0));
		}
		System.out.println("Write User Mem [0-3] = " + achrUserMem[0] + ", " + achrUserMem[1] + ", " + achrUserMem[2] + ", " + achrUserMem[3]);
		intErrorcode = LJUD.ePut(intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chUSER_MEM, 0, achrUserMem);
		ErrorHandler(intErrorcode, 0, new Exception());
		//Re-read the user memory.
		intErrorcode = LJUD.eGet(intHandle[0], LJUD.LJ_ioGET_CONFIG, LJUD.LJ_chUSER_MEM, dummyDouble, achrUserMem);
		ErrorHandler(intErrorcode, 0, new Exception());
		//Display the first 4 elements.
		System.out.println("Read User Mem [0-3] = " + achrUserMem[0] + ", " + achrUserMem[1] + ", " + achrUserMem[2] + ", " + achrUserMem[3]);

	

		//Now a cal constants example.  The calibration memory is passed as doubles.
		//The memory area consists of 8 blocks (0-7) of 16 doubles each, for a total
		//of 128 elements.  As of this writing, block 7 is not used, so we will
		//use the last 4 elements of block 7 for testing, which is elements 124-127.
		//We will read the constants, update a few elements, and write the constants.  
		//The entire memory area is read and written each time.

		//This cal example. Note writing and reading
		//the cal area is an advanced operation.
		//Read the cal constants.
		intErrorcode = LJUD.eGet(intHandle[0], LJUD.LJ_ioGET_CONFIG, LJUD.LJ_chCAL_CONSTANTS, dummyDouble, adblCalMem);
		ErrorHandler(intErrorcode, 0, new Exception());
		//Display the last 4 elements.
		System.out.println("Read Cal Constants [124-127] = " + adblCalMem[124] + ", " + adblCalMem[125] + ", " + adblCalMem[126] + ", " + adblCalMem[127]);
		//Create 4 new pseudo-random numbers to write.  We will update the last
		//4 cal constants, but the rest will be unchanged.
		for(i=124;i<128;i++)
		{
			adblCalMem[i] = (int)(100*((double)rand.nextDouble()/1.0))-50;
		}
		System.out.println("Write  Cal Constants [124-127] = " + adblCalMem[124] + ", " + adblCalMem[125] + ", " + adblCalMem[126] + ", " + adblCalMem[127]);
		//The special value (0x4C6C) must be put in to write the cal constants.
		intErrorcode = LJUD.ePut(intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chCAL_CONSTANTS, 19564, adblCalMem);
		ErrorHandler(intErrorcode, 0, new Exception());
		//Re-read the cal constants.
		intErrorcode = LJUD.eGet(intHandle[0], LJUD.LJ_ioGET_CONFIG, LJUD.LJ_chCAL_CONSTANTS, dummyDouble, adblCalMem);
		ErrorHandler(intErrorcode, 0, new Exception());
		//Display the first 4 elements.
		System.out.println("Read  Cal Constants [124-127] = " + adblCalMem[124] + ", " + adblCalMem[125] + ", " + adblCalMem[126] + ", " + adblCalMem[127]);
	}


	public static void main(String[] args) {
		try {
			new Memory().runExample();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

}