//---------------------------------------------------------------------------
//
//  PWMDutyCycle.java
//  
//	Demonstrates 1 PWM output and 2 duty cycle inputs.
//	Connect FIO0 to FIO1 to FIO2.
//
//  support@labjack.com
//  June 29, 2009
//----------------------------------------------------------------------
//

import java.io.*;
import java.util.Random;
import java.text.*;
import java.util.Scanner;
import com.labjack.labjackud.LJUD; // importing the LJUD class

public class PWMDutyCycle {

	// Loading the LJUDJava dll so that we can use the LJUDJava interface. This
	// example assumes that the dll is in the "../../../LJUDJava/" folder.
	static {
		try {
			System.loadLibrary("../../../LJUDJava/LJUDJava");
		} catch (UnsatisfiedLinkError e) {
			System.err.println("Native code library failed to load.\n" + e);
			System.exit(1);
		}
	}

	// This is our simple error handling function that is called after every UD
	// function call. This function displays the errorcode and string
	// description of the error. It also has an iteration input is useful when 
	// processing results in a loop (getfirst/getnext). A stack trace is also
	// displayed so that you can find the line of code where the error occurred.
	public static void ErrorHandler(int Errorcode, long Iteration, Exception excep) {
		byte[] err = new byte[255];

		if (Errorcode != LJUD.LJE_NOERROR) {
			LJUD.ErrorToString(Errorcode, err);
			System.out.println("Error number = " + Errorcode);
			System.out.println("Error string = " + new String(err));
			System.out.println("Iteration = " + Iteration);
			System.out.println("Stack Trace : ");
			excep.printStackTrace();
			if (Errorcode > LJUD.LJE_MIN_GROUP_ERROR) {
				// Quit if this is a group error.
				System.exit(1);
			}
		}
	}

	public void ReadDutyCycle(int[] intHandle, int timerNumber)
	{
		int intErrorcode;
		double[] dblValue={0};
		double highTime,lowTime,dutyCycle;

		// Variables that satisfy certain method signatures
		int[] dummyInt = {0};
		double[] dummyDouble = {0};

		// Make a number formatter
		DecimalFormat formatter = new DecimalFormat("#0.00");

		//Read from Timer.  We will go ahead and reset the timer (by writing
		//a value of 0) at the same time as the read.  This way if no new
		//edges occur (e.g. 0% or 100% duty cycle) the next read will return
		//the preset high/low times (0/65535 or 65535/0) rather than returning
		//the old values.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioGET_TIMER, timerNumber, 0, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_VALUE, timerNumber, 0, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());
		intErrorcode = LJUD.GoOne(intHandle[0]);
		ErrorHandler(intErrorcode, 0, new Exception());
		intErrorcode = LJUD.GetResult(intHandle[0], LJUD.LJ_ioPUT_TIMER_VALUE, timerNumber, dummyDouble);  //just to check for error
		ErrorHandler(intErrorcode, 0, new Exception());
		intErrorcode = LJUD.GetResult(intHandle[0], LJUD.LJ_ioGET_TIMER, timerNumber, dblValue);
		ErrorHandler(intErrorcode, 0, new Exception());
		//High time is LSW
		highTime = (double)((dblValue[0]) % (65536));
		//Low time is MSW
		lowTime = (double)((dblValue[0]) / (65536));
		//Calculate the duty cycle percentage.
		dutyCycle = 100*highTime/(highTime+lowTime);

		System.out.println("High clicks Timer " + timerNumber + " = " + formatter.format(highTime));
		System.out.println("Low clicks Timer " + timerNumber + " = " + formatter.format(lowTime));
		System.out.println("Duty cycle Timer " + timerNumber + " = " + formatter.format(dutyCycle));
		System.out.println("");
	}

	public void runExample() throws Exception 
	{
		int intErrorcode;
		long lngGetNextIteration;
		int[] intIOType={0}, intChannel={0};
		double[] dblValue={0};
		int[] intHandle={0};
		long resetCounter=1;
		double count=-1;

		// Variables that satisfy certain method signatures
		int[] dummyInt = {0};
		double[] dummyDouble = {0};

		//Open the LabJack UE9.
		intErrorcode = LJUD.OpenLabJack (LJUD.LJ_dtUE9, LJUD.LJ_ctUSB, "1", 1, intHandle);
		//errorcode = OpenLabJack (LJUD.LJ_dtUE9, LJUD.LJ_ctETHERNET, "192.168.1.209", 0, Handle);
		ErrorHandler(intErrorcode, 0, new Exception());


		//Disable all timers and counters to put everything in a known initial state.
		//Disable the timer and counter, and the FIO lines will return to digital I/O.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chNUMBER_TIMERS_ENABLED, 0, 0, 0);
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_COUNTER_ENABLE, 0, 0, 0, 0);
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_COUNTER_ENABLE, 1, 0, 0, 0);
		intErrorcode = LJUD.GoOne(intHandle[0]);


		//Output a PWM output on Timer0 (FIO0) and measure
		//the duty cycle on Timer1 FIO1 and Timer2 FIO2.

		//Use the fixed 750kHz timer clock source.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chTIMER_CLOCK_BASE, LJUD.LJ_tc750KHZ, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Set the divisor to 3 so the actual timer clock is 250kHz.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chTIMER_CLOCK_DIVISOR, 3, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Enable 2 timers.  They will use FIO0 and FIO1.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chNUMBER_TIMERS_ENABLED, 3, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Configure Timer0 as 8-bit PWM.  Frequency will be 250k/256 = 977 Hz.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_MODE, 0, LJUD.LJ_tmPWM8, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Set the PWM duty cycle to 50%.  The passed value is the low time.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_VALUE, 0, 32768, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Configure Timer1 and Timer2 as duty cycle measurement.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_MODE, 1, LJUD.LJ_tmDUTYCYCLE, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_MODE, 2, LJUD.LJ_tmDUTYCYCLE, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());


		//Execute the requests on a single LabJack.  The driver will use a
		//single low-level TimerCounter command to handle all the requests above.
		intErrorcode = LJUD.GoOne(intHandle[0]);
		ErrorHandler(intErrorcode, 0, new Exception());


		//Get all the results just to check for errors.
		intErrorcode = LJUD.GetFirstResult(intHandle[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
		ErrorHandler(intErrorcode, 0, new Exception());
		lngGetNextIteration=0;	//Used by the error handling function.
		while(intErrorcode < LJUD.LJE_MIN_GROUP_ERROR)
		{
			intErrorcode = LJUD.GetNextResult(intHandle[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
			if(intErrorcode != LJUD.LJE_NO_MORE_DATA_AVAILABLE)
			{
				ErrorHandler(intErrorcode, lngGetNextIteration, new Exception());
			}

			lngGetNextIteration++;
		}



		//Set the PWM duty cycle to 25%.  The passed value is the low time.
		intErrorcode = LJUD.ePut(intHandle[0], LJUD.LJ_ioPUT_TIMER_VALUE, 0, 49152, 0);
		ErrorHandler(intErrorcode, 0, new Exception());
		//Now we will reset the duty cycle input timers, so we are sure the
		//reads we do are not old values from before the PWM output was updated.
		Thread.sleep(10);
		intErrorcode = LJUD.ePut(intHandle[0], LJUD.LJ_ioPUT_TIMER_VALUE, 1, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());
		intErrorcode = LJUD.ePut(intHandle[0], LJUD.LJ_ioPUT_TIMER_VALUE, 2, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Wait a little so we are sure a duty cycle measurement has occured.
		Thread.sleep(10);

		//Read from Timer1.
		ReadDutyCycle (intHandle, 1);

		//Read from Timer2.
		ReadDutyCycle (intHandle, 2);

		//Set the PWM duty cycle to 0%.  The passed value is the low time.
		//We are specifying 65535 out of 65536 clicks to be low.  Since
		//this is 8-bit PWM, we actually get 255 low clicks out of 256 total
		//clicks, so the minimum duty cycle is 0.4%.
		intErrorcode = LJUD.ePut(intHandle[0], LJUD.LJ_ioPUT_TIMER_VALUE, 0, 65535, 0);
		ErrorHandler(intErrorcode, 0, new Exception());
		//Now we will reset the duty cycle input timers, so we are sure the
		//reads we do are not old values from before the PWM output was updated.
		Thread.sleep(10);
		intErrorcode = LJUD.ePut(intHandle[0], LJUD.LJ_ioPUT_TIMER_VALUE, 1, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());
		intErrorcode = LJUD.ePut(intHandle[0], LJUD.LJ_ioPUT_TIMER_VALUE, 2, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Wait a little so we are sure a duty cycle measurement has occured.
		Thread.sleep(10);

		//Read from Timer1.
		ReadDutyCycle (intHandle, 1);

		//Read from Timer2.
		ReadDutyCycle (intHandle, 2);




		//Set the PWM duty cycle to 100%.  The passed value is the low time.
		//We are specifying 0 out of 65536 clicks to be low, so the signal
		//will be high the entire time, meaning there are no edges
		//for the input timers to detect, and no measurement should be made.
		intErrorcode = LJUD.ePut(intHandle[0], LJUD.LJ_ioPUT_TIMER_VALUE, 0, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());
		//Now we will reset the duty cycle input timers, so we are sure the
		//reads we do are not old values from before the PWM output was updated.
		Thread.sleep(10);
		intErrorcode = LJUD.ePut(intHandle[0], LJUD.LJ_ioPUT_TIMER_VALUE, 1, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());
		intErrorcode = LJUD.ePut(intHandle[0], LJUD.LJ_ioPUT_TIMER_VALUE, 2, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Wait a little so we are sure a duty cycle measurement has been attempted.
		Thread.sleep(10);

		//Read from Timer1.
		ReadDutyCycle (intHandle, 1);

		//Read from Timer2.
		ReadDutyCycle (intHandle, 2);

		//Disable all timers and counters, and the FIO lines will return to digital I/O.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chNUMBER_TIMERS_ENABLED, 0, 0, 0);
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_COUNTER_ENABLE, 0, 0, 0, 0);
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_COUNTER_ENABLE, 1, 0, 0, 0);
		intErrorcode = LJUD.GoOne(intHandle[0]);
	}


	public static void main(String[] args) {
		try {
			new PWMDutyCycle().runExample();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

}