//---------------------------------------------------------------------------
//
//  Quadrature.java
//  
//	Enables quadrature input on FIO0/FIO1 and reads the current count
//	every half second, until a key is pressed.  To make something happen
//	without an actual quadrature signal, connect a couple wires to
//	ground and tap them on FIO0 and FIO1.
//
//  support@labjack.com
//  June 29, 2009
//----------------------------------------------------------------------
//

import java.io.*;
import java.text.*;
import java.util.Scanner;
import com.labjack.labjackud.LJUD; // importing the LJUD class

public class Quadrature {

	// Loading the LJUDJava dll so that we can use the LJUDJava interface. This
	// example assumes that the dll is in the "../../../LJUDJava/" folder.
	static {
		try {
			System.loadLibrary("../../../LJUDJava/LJUDJava");
		} catch (UnsatisfiedLinkError e) {
			System.err.println("Native code library failed to load.\n" + e);
			System.exit(1);
		}
	}

	// This is our simple error handling function that is called after every UD
	// function call. This function displays the errorcode and string
	// description of the error. It also has an iteration input is useful when 
	// processing results in a loop (getfirst/getnext). A stack trace is also
	// displayed so that you can find the line of code where the error occurred.
	public static void ErrorHandler(int Errorcode, long Iteration, Exception excep) {
		byte[] err = new byte[255];

		if (Errorcode != LJUD.LJE_NOERROR) {
			LJUD.ErrorToString(Errorcode, err);
			System.out.println("Error number = " + Errorcode);
			System.out.println("Error string = " + new String(err));
			System.out.println("Iteration = " + Iteration);
			System.out.println("Stack Trace : ");
			excep.printStackTrace();
			if (Errorcode > LJUD.LJE_MIN_GROUP_ERROR) {
				// Quit if this is a group error.
				System.exit(1);
			}
		}
	}

	// Java uses signed bytes but the UD driver requires unsigned bytes therefore these two methods
	// are needed to convert back and forth. For more info see http://www.lykkenborg.no/java/2005/03/how-do-i-read-unsigned-byte-in-java.html
	public static byte signed(int i) 
	{
		return (byte) i;
	}

	public static int unsigned(byte b) 
	{
		return b & 0xFF;
	}

	public void runExample() throws Exception 
	{
		int intErrorcode;
		long lngGetNextIteration;
		int[] intIOType={0}, intChannel={0};
		double[] dblValue={0};
		int[] intHandle={0};
		boolean finished = false;
		Scanner s = new Scanner(System.in);

		//Open the LabJack UE9.
		intErrorcode = LJUD.OpenLabJack (LJUD.LJ_dtUE9, LJUD.LJ_ctUSB, "1", 1, intHandle);
		//intErrorcode = LJUD.OpenLabJack (LJUD.LJ_dtUE9, LJUD.LJ_ctETHERNET, "192.168.1.209", 0, intHandle);
		ErrorHandler(intErrorcode, 0, new Exception());

		// Variables to satisfy certain method signatures
		double[] dummyDouble = {0};
		int[] dummyInt = {0};

		//Disable all timers and counters to put everything in a known initial state.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chNUMBER_TIMERS_ENABLED, 0, 0, 0);
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_COUNTER_ENABLE, 0, 0, 0, 0);
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_COUNTER_ENABLE, 1, 0, 0, 0);
		intErrorcode = LJUD.GoOne(intHandle[0]);
		ErrorHandler(intErrorcode, 0, new Exception());


		//First enable the quadrature input.

		//Enable 2 timers for phases A and B.  They will use FIO0 and FIO1.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chNUMBER_TIMERS_ENABLED, 2, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Configure Timer0 as quadrature.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_MODE, 0, LJUD.LJ_tmQUAD, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Configure Timer1 as quadrature.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_MODE, 1, LJUD.LJ_tmQUAD, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Execute the requests on a single LabJack.  The driver will use a
		//single low-level TimerCounter command to handle all the requests above.
		intErrorcode = LJUD.GoOne(intHandle[0]);
		ErrorHandler(intErrorcode, 0, new Exception());



		//Get all the results just to check for errors.
		intErrorcode = LJUD.GetFirstResult(intHandle[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
		ErrorHandler(intErrorcode, 0, new Exception());
		lngGetNextIteration=0;	//Used by the error handling function.
		while(intErrorcode < LJUD.LJE_MIN_GROUP_ERROR)
		{
			intErrorcode = LJUD.GetNextResult(intHandle[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
			if(intErrorcode != LJUD.LJE_NO_MORE_DATA_AVAILABLE)
			{
				ErrorHandler(intErrorcode, lngGetNextIteration, new Exception());
			}
			lngGetNextIteration++;
		}



		while(!finished)
		{

			//Wait 500 milliseconds
			Thread.sleep(500);

			//Request a read from Timer0.  Timer0 and Timer1 both return the same
			//quadrature value.
			intErrorcode = LJUD.eGet(intHandle[0], LJUD.LJ_ioGET_TIMER, 0, dblValue, 0);
			ErrorHandler(intErrorcode, 0, new Exception());
			System.out.println("Quad Counter = " + dblValue[0]);
			System.out.println("\nPress enter to go again or q to quit");
			finished = s.nextLine().equals("q");
		}


		//Disable the timers and the FIO lines will return to digital I/O.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chNUMBER_TIMERS_ENABLED, 0, 0, 0);
		intErrorcode = LJUD.GoOne(intHandle[0]);
	}


	public static void main(String[] args) {
		try {
			new Quadrature().runExample();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

}