//---------------------------------------------------------------------------
//
//  Quadrature.java
//  
//	Enables quadrature input on FIO0/FIO1 and reads the current count
//	every half second, until a key is pressed.  To make something happen
//	without an actual quadrature signal, connect a couple wires to
//	ground and tap them on FIO0 and FIO1.
//
//  support@labjack.com
//  June 29, 2009
//----------------------------------------------------------------------
//

import java.io.*;
import java.text.*;
import java.util.Scanner;
import com.labjack.labjackud.LJUD; // importing the LJUD class

public class RawIO {

	// Loading the LJUDJava dll so that we can use the LJUDJava interface. This
	// example assumes that the dll is in the "../../../LJUDJava/" folder.
	static {
		try {
			System.loadLibrary("../../../LJUDJava/LJUDJava");
		} catch (UnsatisfiedLinkError e) {
			System.err.println("Native code library failed to load.\n" + e);
			System.exit(1);
		}
	}

	// This is our simple error handling function that is called after every UD
	// function call. This function displays the errorcode and string
	// description of the error. It also has an iteration input is useful when 
	// processing results in a loop (getfirst/getnext). A stack trace is also
	// displayed so that you can find the line of code where the error occurred.
	public static void ErrorHandler(int Errorcode, long Iteration, Exception excep) {
		byte[] err = new byte[255];

		if (Errorcode != LJUD.LJE_NOERROR) {
			LJUD.ErrorToString(Errorcode, err);
			System.out.println("Error number = " + Errorcode);
			System.out.println("Error string = " + new String(err));
			System.out.println("Iteration = " + Iteration);
			System.out.println("Stack Trace : ");
			excep.printStackTrace();
			if (Errorcode > LJUD.LJE_MIN_GROUP_ERROR) {
				// Quit if this is a group error.
				System.exit(1);
			}
		}
	}

	// Java uses signed bytes but the UD driver requires unsigned bytes therefore these two methods
	// are needed to convert back and forth. For more info see http://www.lykkenborg.no/java/2005/03/how-do-i-read-unsigned-byte-in-java.html
	public static byte signed(int i) 
	{
		return (byte) i;
	}

	public static int unsigned(byte b) 
	{
		return b & 0xFF;
	}

	public void runExample() throws Exception 
	{
		int intErrorcode;
		int[] intHandle={0};
		char ch;
		long time=0, i=0;
		long numIterations = 1000;

		double[] numBytesToWrite = new double[1];
		double[] numBytesToRead = new double[1];
		byte[] writeArray = new byte[2];
		writeArray[0] = 112;
		writeArray[1] = 112;
		byte[] readArray = {0};


		//Open the first found LabJack UE9.
		intErrorcode = LJUD.OpenLabJack (LJUD.LJ_dtUE9, LJUD.LJ_ctUSB, "1", 1, intHandle);
		//intErrorcode = LJUD.OpenLabJack (LJUD.LJ_dtUE9, LJUD.LJ_ctETHERNET, "192.168.1.209", 0, intHandle);
		ErrorHandler(intErrorcode, 0, new Exception());

		time = System.currentTimeMillis();

		for(i=0;i<numIterations;i++)
		{
			numBytesToWrite[0] = 2;
			numBytesToRead[0] = 2;

			//Raw Out
			intErrorcode = LJUD.eGet(intHandle[0], LJUD.LJ_ioRAW_OUT, 0, numBytesToWrite, writeArray);
			ErrorHandler(intErrorcode, 0, new Exception());

			//Raw In
			intErrorcode = LJUD.eGet(intHandle[0], LJUD.LJ_ioRAW_IN, 0, numBytesToRead, readArray);
			ErrorHandler(intErrorcode, 0, new Exception());

		}

		time = System.currentTimeMillis() - time;

		System.out.println("Milleseconds per iteration = " + (double)time / (double)numIterations);
	}


	public static void main(String[] args) {
		try {
			new RawIO().runExample();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

}